<?xml version="1.0" encoding="UTF-8"  ?>

<xsl:stylesheet version="1.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:xpe="http://www.dsi.unive.it/~maffei/pear"
  xmlns="http://www.w3.org/1999/xhtml">
  
  <xsl:output 
   method="html"
   version="1.0"
   encoding="UTF-8"
   indent="yes"
   media-type="text/html"
   doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN"
   omit-xml-declaration="no" />

  <xsl:strip-space elements="xpe:*"/>
  
  <xsl:template match="xpe:protocol-encoding">
    <xsl:variable name="name" select="xpe:protocol/@name"/>
    <xsl:comment>
      <xsl:text> </xsl:text> <xsl:value-of select="$name"/> <xsl:text>: proof tree generated by proof-tree.xsl stylesheet </xsl:text>
    </xsl:comment>
    <xsl:text>
</xsl:text>
    <html>
      <head>
        <title> <xsl:value-of select="$name"/> <xsl:text> Proof Tree</xsl:text> </title>
        <style type="text/css">
          html {
            margin: 0px;
            padding: 0px;
          }
          
          body {
            margin: 0px;
            padding: 5px;
            font-family: Geneva, Verdana, Helvetica, Arial, serif;
            font-size: 12px;
          }
          
          h1 {
            font-size: 16px;
            margin-top: 10px;
          }
          h2 {
            font-size: 14px;
            margin-top: 30px;
            margin-bottom: 15px;
          }
          
          #spedef {
            margin-left: 10px;
            margin-bottom: 17px;
            padding-bottom: 2px;
          }
          
          #instr {
            font-style: italic;
            padding-top: 2px;
            padding-right: 6px;
            border-right: 1px solid #000000;
            border-bottom: 1px solid #000000;
          }
          
          #containers {
            padding-top: 2px;
            padding-bottom: 6px;
            padding-left: 6px;
            padding-right: 6px;	
            border-bottom: 1px solid #000000;
          }
          
          #rule {
            padding-top: 8px;
            padding-bottom: 30px;
            padding-left: 0px;
            padding-right: 10px;
          }
          
          #cond {
            padding-top: 8px;
            padding-bottom: 30px;
            padding-right: 20px;
            padding-left: 6px;
          }
          
          #act {
            padding-top: 8px;
            padding-bottom: 30px;
            padding-right: 10px;
            padding-left: 6px;
          }
          
          #seqproc {
            border: 1px none #000000;
            margin-left: 0px;
            margin-top: 25px;
          }
          
          #nil {
            width: 600px;
            border: 1px none #000000;
            margin-left: 10px;
            margin-bottom: 15px;
          }
        </style>
      </head>
      <body> <xsl:apply-templates/> </body>
    </html>
  </xsl:template>
  
  <xsl:template match="xpe:associated-rules" />
  <xsl:template match="xpe:rule-system" />
  <xsl:template match="xpe:description" />
  
  <xsl:template match="xpe:protocol">
    <div id="title">
      <h1> <xsl:text>Protocol </xsl:text> <xsl:value-of select="@name"/> </h1>
    </div>
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="xpe:signature">
    <div id="seqproc">
      <h2>
        <xsl:value-of select="@principal" /> <xsl:text> &gt; </xsl:text> <xsl:value-of select="../@name" />
        <xsl:text>(</xsl:text> <xsl:apply-templates /> <xsl:text>) :=</xsl:text>
      </h2>
    </div>
  </xsl:template>
  
  <xsl:template match="xpe:declaration">
    <xsl:value-of select="@identifier" />
    <xsl:text>:</xsl:text> <xsl:call-template name="id-type" /> <xsl:apply-templates />
    <xsl:call-template name="comma" />
  </xsl:template>
  
  <xsl:template match="xpe:key-type">
    <xsl:text>(</xsl:text>
    <xsl:for-each select="xpe:identifier"> <xsl:value-of select="@name" /> <xsl:call-template name="comma" /> </xsl:for-each>
    <xsl:text>)</xsl:text>
  </xsl:template>
  
  <xsl:template match="xpe:definition">
    <div id="spdef">
      <table cellspacing="0">
        <xsl:apply-templates />
      </table>
    </div>
  </xsl:template>
  
  <xsl:template match="xpe:instruction-node">
    <tr id="instrow">
      <xsl:apply-templates select="xpe:instruction" />
      <xsl:apply-templates select="xpe:containers" />
    </tr>
    <tr id="rulerow">
       <xsl:apply-templates select="xpe:rule" />
    </tr>
  </xsl:template>
  
  <xsl:template match="xpe:instruction">
    <xsl:variable name="opname" select="ancestor::xpe:operation[1]/@name" />
    <xsl:variable name="ctname" select="ancestor::xpe:container[1]/@name" />
    <xsl:choose>
      <xsl:when test="$opname != '' or $ctname != ''">
        <xsl:call-template name="instr-body" />
        <xsl:call-template name="comma" />
      </xsl:when>
      <xsl:otherwise>
        <td id="instr" nowrap="nowrap"> <xsl:call-template name="instr-body" /> </td>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="xpe:containers">
    <td id="containers" nowrap="nowrap" colspan="2">
      <xsl:apply-templates />
    </td>
  </xsl:template> 
  
  <xsl:template match="xpe:container">
    <xsl:value-of select="@name" /> <xsl:text>: </xsl:text> <xsl:apply-templates /> <br />
  </xsl:template>


  <xsl:template match="xpe:rule">
    <td id="rule"> <xsl:text>(</xsl:text> <xsl:value-of select="@name" /> <xsl:text>)</xsl:text> </td>
    <xsl:apply-templates />
  </xsl:template>

  <xsl:template match="xpe:conditions">
    <td id="cond" nowrap="nowrap" > <xsl:apply-templates /> </td>
  </xsl:template>
  
  <xsl:template match="xpe:condition">
    <xsl:apply-templates /> <xsl:call-template name="newline" />
  </xsl:template>
  
  <xsl:template match="xpe:ipothesis">
    <xsl:apply-templates />
    <xsl:text> -&gt; </xsl:text> <br/>
  </xsl:template>  

  <xsl:template match="xpe:thesis">
    <xsl:apply-templates /> <br />
  </xsl:template>  

  <xsl:template match="xpe:bool">
    <xsl:variable name="nnot" select="@type != 'not'" />
    <xsl:if test="$nnot"> <xsl:text>(</xsl:text> </xsl:if>
      <xsl:apply-templates />
    <xsl:if test="$nnot"> <xsl:text>)</xsl:text> </xsl:if>
    <xsl:call-template name="bool-op" />
  </xsl:template>
  
  <xsl:template match="xpe:operation">
    <xsl:if test="../@type = 'not'"> <xsl:text>~</xsl:text> </xsl:if>
    <xsl:call-template name="op" />
    <xsl:call-template name="bool-op" />
    <xsl:call-template name="newline" />
  </xsl:template>

  <xsl:template match="xpe:message">
    <xsl:choose>
      <xsl:when test="../@name='encrypt'">
        <xsl:text> </xsl:text><xsl:call-template name="message-body" /> <xsl:text> as </xsl:text><xsl:value-of select="@name" />
      </xsl:when>
      <xsl:when test="../@name='decrypt'">
        <xsl:text> </xsl:text><xsl:value-of select="@name" /><xsl:text> as </xsl:text> <xsl:call-template name="message-body" />
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="name" select="@name"/>
        <xsl:if test="$name"> <xsl:value-of select="$name" /> <xsl:text> = </xsl:text> </xsl:if> <xsl:call-template name="message-body" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="xpe:identifier">
    <xsl:value-of select="@name" />
    <xsl:if test="@tag">:<xsl:value-of select="@tag" /></xsl:if>
    <xsl:call-template name="comma" />
    <xsl:apply-templates />
  </xsl:template>
  
  <xsl:template match="xpe:tag-parameters">
    <xsl:text>(</xsl:text> <xsl:apply-templates /> <xsl:text>)</xsl:text>
  </xsl:template>
  
  <xsl:template match="xpe:tagged-identifiers">
    <xsl:text>{</xsl:text> <xsl:apply-templates /> <xsl:text>}:</xsl:text>
    <xsl:value-of select="@tag" />
    <xsl:call-template name="comma" />
  </xsl:template>
  
  <xsl:template match="xpe:valued-identifier">
    <xsl:value-of select="@name" /> <xsl:text> = </xsl:text> <xsl:apply-templates />
    <xsl:call-template name="comma" />
  </xsl:template>

  <xsl:template match="xpe:actions">
    <td id="act">
      <xsl:if test="position() = 1"> <xsl:attribute name="colspan">2</xsl:attribute> </xsl:if>
      <xsl:for-each select="xpe:operation"> <xsl:call-template name="op" /> <br /> </xsl:for-each>
    </td>
  </xsl:template>
  
  <xsl:template match="xpe:null-node">
    <tr id="nil">
      <xsl:apply-templates select="xpe:nil" />
      <xsl:apply-templates select="xpe:containers" />
    </tr>
    <tr>
       <xsl:apply-templates select="xpe:rule" />
    </tr>
  </xsl:template>
  
  <xsl:template match="xpe:nil">
    <td id="instr"> <xsl:text>0</xsl:text> </td>
    <xsl:apply-templates />
  </xsl:template>
  
  <xsl:template match="xpe:instantiation">
    <div id="inst">
      <h2>
        <xsl:text>start.</xsl:text> <xsl:apply-templates />
      </h2>
    </div>
  </xsl:template>
  
  <xsl:template match="xpe:key-binding">
    <xsl:text>let </xsl:text><xsl:value-of select="@key" /><xsl:text> = </xsl:text><xsl:call-template name="id-type" />
    <xsl:apply-templates />
    <xsl:text>.</xsl:text>
  </xsl:template>
  
  <xsl:template match="xpe:sequential-process-calls">
    <br/> <xsl:text>(</xsl:text> <xsl:apply-templates /> <xsl:text>)</xsl:text>
  </xsl:template>
  
  <xsl:template match="xpe:call">
    <xsl:value-of select="@principal" /><xsl:text> &gt;</xsl:text><xsl:if test="@replicated='true'">!</xsl:if>
    <xsl:text> </xsl:text><xsl:value-of select="@sequential-process" />
    <xsl:text>(</xsl:text><xsl:apply-templates /><xsl:text>)</xsl:text>
    <xsl:call-template name="vert" />
  </xsl:template>
  
  <!-- named templates -->
  
  <xsl:template name="newline"> <xsl:if test="position() != last()"> <br /> </xsl:if> </xsl:template>
  <xsl:template name="and"> <xsl:if test="position() != last()"> <xsl:text> &amp;</xsl:text> </xsl:if> </xsl:template>
  <xsl:template name="or"> <xsl:if test="position() != last()"> <xsl:text> |</xsl:text> </xsl:if> </xsl:template>
  <xsl:template name="comma"><xsl:if test="position() != last()"> <xsl:text>, </xsl:text> </xsl:if> </xsl:template>
  <xsl:template name="vert"> <xsl:if test="position() != last()"> <xsl:text> | </xsl:text> </xsl:if> </xsl:template>
  
  <xsl:template name="vbar">
    <xsl:if test="../descendant-or-self::xpe:message/@encryption = 'public' or @encryption = 'private'"> <xsl:text>|</xsl:text> </xsl:if>
  </xsl:template>

  <xsl:template name="key">
    <xsl:param name="msg" select="ancestor-or-self::xpe:message" />
    <xsl:choose>
      <xsl:when test="$msg/@encryption = 'public'"> <xsl:text>Pub(</xsl:text> <xsl:value-of select="$msg/@key" /> <xsl:text>)</xsl:text> </xsl:when>
      <xsl:when test="$msg/@encryption = 'private'"> <xsl:text>Priv(</xsl:text> <xsl:value-of select="$msg/@key" /> <xsl:text>)</xsl:text> </xsl:when>
      <xsl:when test="$msg/@encryption = 'symmetric'"> <xsl:value-of select="$msg/@key" /> </xsl:when>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="message-body">
    <xsl:text>{</xsl:text> <xsl:call-template name="vbar" /><xsl:apply-templates /> <xsl:call-template name="vbar" /> <xsl:text>}</xsl:text>
    <xsl:if test="@encryption != 'none' and @key != ''"> <xsl:text>_</xsl:text> <xsl:call-template name="key" /> </xsl:if>
  </xsl:template>
  
  <xsl:template name="id-type">
    <xsl:choose>
      <xsl:when test="xpe:key-type/@type='symmetric'"><xsl:text>sym-key</xsl:text></xsl:when>
      <xsl:when test="xpe:key-type/@type='asymmetric'"><xsl:text>asym-key</xsl:text></xsl:when>
      <xsl:when test="@type='Id'"><xsl:text>Id</xsl:text></xsl:when>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="op">
    <xsl:value-of select="@container" /> <xsl:text>.</xsl:text>
    <xsl:value-of select="@name" /> <xsl:text>(</xsl:text> <xsl:apply-templates /> <xsl:text>)</xsl:text>
  </xsl:template>
  
  <xsl:template name="bool-op">
    <xsl:choose>
      <xsl:when test="../@type = 'or'"> <xsl:call-template name="or" /> </xsl:when>
      <xsl:otherwise> <xsl:call-template name="and" /> </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="instr-body">
    <xsl:variable name="id" select="@identifier"/>
    <xsl:if test="$id"> <xsl:value-of select="$id" /> <xsl:text> = </xsl:text> </xsl:if>
    <xsl:variable name="instr" select="@name"/>
    <xsl:value-of select="$instr" />
    <xsl:choose>
      <xsl:when test="$instr='encrypt' or $instr='decrypt'"> <xsl:apply-templates /> </xsl:when>
      <xsl:otherwise> <xsl:text>(</xsl:text> <xsl:apply-templates /> <xsl:text>)</xsl:text> </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
</xsl:stylesheet>