<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet version="1.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xpe="http://www.dsi.unive.it/~maffei/pear">

<xsl:output
    method="text"
    version="1.0"
    encoding="UTF-8"
    indent="no"
    omit-xml-declaration="yes"
    media-type="text/plain"/>

<xsl:strip-space elements="xpe:*"/>

<xsl:template match="xpe:protocol-encoding">
  <xsl:text>// </xsl:text><xsl:value-of select="xpe:protocol/@name"/>
  <xsl:text>: protocol encoding generated by rho-spi.xsl stylesheet
  </xsl:text>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="xpe:associated-rules">
  <xsl:text>
@rules: </xsl:text><xsl:apply-templates/>
</xsl:template>

<xsl:template match="xpe:rule-system">
  <xsl:text>"</xsl:text><xsl:value-of select="@name" /><xsl:text>"</xsl:text><xsl:call-template name="comma" />
  <xsl:apply-templates />
</xsl:template>

<xsl:template match="xpe:description">
  <xsl:text>

@desc[</xsl:text><xsl:apply-templates/><xsl:text>]
</xsl:text>
</xsl:template>

<xsl:template match="xpe:protocol">
  <xsl:text>
Protocol </xsl:text><xsl:value-of select="@name"/><xsl:text>:
</xsl:text>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="xpe:signature">
  <xsl:text>
  </xsl:text><xsl:value-of select="@principal" /><xsl:text> &gt; </xsl:text><xsl:value-of select="../@name" />
  <xsl:text>(</xsl:text><xsl:apply-templates /><xsl:text>) :=
</xsl:text>
</xsl:template>

<xsl:template match="xpe:declaration">
  <xsl:value-of select="@identifier" />
  <xsl:text>:</xsl:text><xsl:call-template name="id-type" /><xsl:apply-templates />
  <xsl:call-template name="comma" />
</xsl:template>

<xsl:template match="xpe:key-type">
  <xsl:text>(</xsl:text>
  <xsl:for-each select="xpe:identifier"><xsl:value-of select="@name" /><xsl:call-template name="comma" /></xsl:for-each>
  <xsl:text>)</xsl:text>
</xsl:template>

<xsl:template match="xpe:instruction">
  <xsl:variable name="label" select="../@label" />
  <xsl:choose>
    <xsl:when test="$label"> <xsl:value-of select="$label" /> <xsl:text>:  </xsl:text> </xsl:when>
    <xsl:otherwise> <xsl:text>    </xsl:text> </xsl:otherwise>
  </xsl:choose>
  <xsl:value-of select="@name" />
  <xsl:choose>
    <xsl:when test="@name='encrypt' or @name='decrypt'"> <xsl:apply-templates /> </xsl:when>
    <xsl:otherwise> <xsl:text>(</xsl:text> <xsl:apply-templates /> <xsl:text>)</xsl:text> </xsl:otherwise>
  </xsl:choose>
  <xsl:text>.
</xsl:text>
</xsl:template>

<xsl:template match="xpe:message">
  <xsl:choose>
    <xsl:when test="../@name='encrypt'">
      <xsl:text> </xsl:text><xsl:call-template name="message-body" /><xsl:text> as </xsl:text><xsl:value-of select="@name" />
    </xsl:when>
    <xsl:when test="../@name='decrypt'">
      <xsl:text> </xsl:text><xsl:value-of select="@name" /><xsl:text> as </xsl:text><xsl:call-template name="message-body" />
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="@name" /><xsl:text> = </xsl:text><xsl:call-template name="message-body" />
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="xpe:identifier">
  <xsl:value-of select="@name" />
  <xsl:if test="@tag">:<xsl:value-of select="@tag" /></xsl:if>
  <xsl:call-template name="comma" />
  <xsl:apply-templates />
</xsl:template>

<xsl:template match="xpe:tag-parameters">
  <xsl:text>(</xsl:text> <xsl:apply-templates /> <xsl:text>)</xsl:text>
</xsl:template>

<xsl:template match="xpe:tagged-identifiers">
  <xsl:text>{</xsl:text> <xsl:apply-templates /> <xsl:text>}:</xsl:text>
  <xsl:value-of select="@tag" />
  <xsl:call-template name="comma" />
</xsl:template>

<xsl:template match="xpe:rule" />
<xsl:template match="xpe:containers" />

<xsl:template match="xpe:nil">
  <xsl:text>    0
</xsl:text>
  <xsl:apply-templates />
</xsl:template>

<xsl:template match="xpe:instantiation">
  <xsl:text>
start.</xsl:text>
  <xsl:apply-templates />
</xsl:template>

<xsl:template match="xpe:key-binding">
  <xsl:text>
  let </xsl:text><xsl:value-of select="@key" /><xsl:text> = </xsl:text><xsl:call-template name="id-type" />
  <xsl:apply-templates />.
</xsl:template>

<xsl:template match="xpe:sequential-process-calls">
  <xsl:text>( </xsl:text><xsl:apply-templates /><xsl:text>)</xsl:text>
</xsl:template>

<xsl:template match="xpe:call">
  <xsl:value-of select="@principal" /><xsl:text> &gt;</xsl:text><xsl:if test="@replicated='true'">!</xsl:if>
  <xsl:text> </xsl:text><xsl:value-of select="@sequential-process" />
  <xsl:text>(</xsl:text><xsl:apply-templates /><xsl:text>)
</xsl:text>
  <xsl:call-template name="vert" />
</xsl:template>

<!-- functions -->

<xsl:template name="comma"> <xsl:if test="position() != last()"> <xsl:text>, </xsl:text> </xsl:if> </xsl:template>
<xsl:template name="vert"> <xsl:if test="position() != last()"> <xsl:text>| </xsl:text> </xsl:if> </xsl:template>

<xsl:template name="vbar"> <xsl:if test="@encryption = 'public' or @encryption = 'private'"> <xsl:text>|</xsl:text> </xsl:if> </xsl:template>

<xsl:template name="key">
  <xsl:choose>
    <xsl:when test="@encryption = 'public'"> <xsl:text>Pub(</xsl:text> <xsl:value-of select="@key" /> <xsl:text>)</xsl:text> </xsl:when>
    <xsl:when test="@encryption = 'private'"> <xsl:text>Priv(</xsl:text> <xsl:value-of select="@key" /> <xsl:text>)</xsl:text> </xsl:when>
    <xsl:when test="@encryption = 'symmetric'"> <xsl:value-of select="@key" /> </xsl:when>
  </xsl:choose>
</xsl:template>

<xsl:template name="message-body">
  <xsl:text>{</xsl:text> <xsl:call-template name="vbar" /> <xsl:apply-templates /> <xsl:call-template name="vbar" /> <xsl:text>}</xsl:text>
  <xsl:if test="@encryption != 'none' and @key != ''"> <xsl:text>_</xsl:text> <xsl:call-template name="key" /> </xsl:if>
</xsl:template>

<xsl:template name="id-type">
  <xsl:variable name="keyType" select="xpe:key-type/@type" />
  <xsl:choose>
    <xsl:when test="$keyType='symmetric'"><xsl:text>sym-key</xsl:text></xsl:when>
    <xsl:when test="$keyType='asymmetric'"><xsl:text>asym-key</xsl:text></xsl:when>
    <xsl:when test="@type='Id'"><xsl:text>Id</xsl:text></xsl:when>
  </xsl:choose>
</xsl:template>

</xsl:stylesheet>