/*
 * MapContainer.java
 *
 * Created on 20 giugno 2004, 16.34
 */

package it.unive.dsi.pear.plugins;

import it.unive.dsi.pear.core.parser.Data;
import it.unive.dsi.pear.core.parser.ValueData;
import it.unive.dsi.pear.core.parser.ValueDataImpl;
import it.unive.dsi.pear.util.ArrayIterator;
import it.unive.dsi.pear.util.CloneableIterator;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;


/** This class implements a container that associates a value to each element.
 * @author Francesco Placella
 */
public class MapContainer extends SetContainer
{
    protected Map infos = new HashMap();
    
    public boolean contains(Data data) { return container.containsKey(data.getInfo()); }
    public Collection content() { return container.values(); }
    public CloneableIterator infoIterator() { return new ArrayIterator(infos.values().toArray()); }
    
    public String getContentImage() { return this.content().toString(); }
    
    public boolean add(Data data) throws OperationException
    {
        return this.add(data, null);
    }

    public boolean add(Data key, Data data) throws OperationException
    {
        return this.add(key.getInfo(), this.newElement(key, data));
    }

    public final Data set(Data data) { return this.set(data, null); }

    public Data set(Data key, Data data)
    {
        String info = key.getInfo();
        if (container.get(info) != null) 
            return (Data)container.put(info, this.newElement(key, data));
        else
            return null;
    }

    public Data get(Data key)
    {
        ValueData vd = (ValueData)container.get(key.getInfo());
        if (vd == null) return null;
        return vd.getDataValue();
    }
    
    public boolean autoPatternMatch(Data arg, Data data, Map substMap)
    {
        Data d = (Data)infos.get(data.getInfo());
        return super.autoPatternMatch(arg, data, substMap) ||
               super.autoPatternMatch(arg, this.get(data), substMap) ||
               super.autoPatternMatch(arg, d, substMap);
    }
    
    protected Data newElement(Data key, Data value)
    {
        String info = key.getInfo();
        infos.put(info, key.newData(info));
        return new ValueDataImpl(info, value);
    }
}
