/*
 * OrdContainer.java
 *
 * Created on 8 marzo 2004, 14.32
 */

package it.unive.dsi.pear.plugins;

import java.util.*;

import it.unive.dsi.pear.core.parser.Data;
import it.unive.dsi.pear.core.AbstractContainer;

/**
 * This class implements an ordered container: each element has an integer 
 * index associated.
 * @author  Francesco Placella
 */
public class OrdContainer extends AbstractContainer
{
    private HashMap infoMap = new HashMap();
    protected LinkedList container = new LinkedList();
    private int curIndex;
    
    public Object clone()
    {
        OrdContainer cloned = (OrdContainer)super.clone();
        cloned.container = (LinkedList)container.clone();
        cloned.infoMap = (HashMap)infoMap.clone();
        return cloned;
    }
    
    public Data set(Data data)
    {
        curIndex = this.indexOfInfo(data);
        if (curIndex != -1) return (Data)container.set(curIndex, data);
        else return null;
    }
    
    public final boolean add(Data data) throws OperationException
    {
        return this.add(data.getInfo(), data);
    }
    
    public final boolean add(Data key, Data data) throws OperationException
    {
        return this.add(key.getInfo(), data);
    }

    private boolean add(Object info, Data data) throws OperationException
    {
        if (infoMap.containsKey(info))
            throw new OperationException("data already contained");

        infoMap.put(info, data);
        curIndex = container.size();
        return container.add(data);
    }
    
    
    public void clear()
    {
        container.clear();
        infoMap.clear();
        curIndex = -1;
    }
    
    public Collection content() { return container; }
    
    public boolean delete(Data data)
    {
        curIndex = container.size() - 1;
        infoMap.remove(data.getInfo());
        return container.remove(data);
    }
    
    public boolean contains(Data data)
    {
        return this.indexOf(data) != -1;
    }
    
    public int indexOf(Object obj) { return curIndex = container.indexOf(obj); }
    
    private int indexOfInfo(Data data)
    {
        ListIterator it = container.listIterator();
        while (it.hasNext())
        {
            int index = it.nextIndex();
            Data d = (Data)it.next();
            if (d != null && data.getInfo().equals(d.getInfo()))
                return index;
        }
        return -1;
    }

    public Data get(Data data) { return (Data)infoMap.get(data.getInfo()); }
    
    /** Getter for property curIndex.
     * @return Value of property curIndex.
     *
     */
    public int getCurIndex() { return curIndex; }
}
