/*
 * AbstractContainer.java
 *
 * Created on 29 giugno 2004, 17.01
 */

package it.unive.dsi.pear.core;

import java.util.*;

import it.unive.dsi.pear.util.*;

import it.unive.dsi.pear.core.parser.Container;
import it.unive.dsi.pear.core.parser.Data;

/** This class is a standard implementation of the interface
 * <CODE>Container</CODE>.
 * @author Francesco Placella
 */
abstract public class AbstractContainer implements Container
{
    private String name;
    
    /** Creates and returns a copy of this object.
     * @return a copy of this object
     */    
    public Object clone()
    {
        try { return super.clone(); }
        catch (CloneNotSupportedException cnse) { return null; }
    }

    /** Returns a cloneable iterator over the elements in this container.
     * @return an Iterator over the elements in this collection
     */    
    public CloneableIterator infoIterator()
    {
        return new ArrayIterator(this.toArray());
    }

    /** Returns an array containing all of the elements in this collection.
     * @return an array containing all of the elements in this collection
     */    
    public final Object[] toArray() { return this.content().toArray(); }

    /** Returns a string representation of the object.
     * @return a string representation of the object
     */    
    public String toString()
    {
        return this.getName(); //  + ": " + this.content().toString();
    }
    
    /** Performs an auto pattern-matching between the first argument and the
     * second. It should be overridden to match the logic of the actual 
     * container.
     * @param arg the data to be matched
     * @param data the data to be checked for matching
     * @param substMap the substitution map
     * @return <CODE>true</CODE> if the matching has been successful,
     * <CODE>false</CODE> otherwise.
     */    
    public boolean autoPatternMatch(Data arg, Data data, Map substMap)
    {
        return arg.autoSubst(data, substMap);
    }
    
    /** Returns the number of elements in this container.
     * @return the size of the container
     */    
    public int size() { return this.content().size(); }
    
    /** Returns a string representation of the content of the container.
     * @return a string representation of the content of the container
     */    
    public String getContentImage() { return this.content().toString(); }

    
    /** Getter for property name.
     * @return Value of property name.
     *
     */
    public java.lang.String getName() {
        return name;
    }
    
    /** Setter for property name.
     * @param name New value of property name.
     *
     */
    public void setName(java.lang.String name) {
        this.name = name;
    }
}
