/*
 * Configuration.java
 *
 * Created on 7 marzo 2004, 23.11
 */

package it.unive.dsi.pear.core;

import it.unive.dsi.pear.plugins.tabber.TabPlugin;
import it.unive.dsi.pear.util.JarClassLoader;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringWriter;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;



/**
 *
 * @author  Francesco Placella
 */
public final class Configuration
{
    private static final String[] NO_RESOURCES = {};
    
    private static final String VR_EXT = ".vr";
    private static final String PRT_EXT = ".rospi";
    private static final String PLUGINS_PACKAGE = "it.unive.dsi.pear.plugins.";
    private static final String TABBER_PACKAGE = "it.unive.dsi.pear.plugins.tabber.";
    private static final String PLUGINS_DIR = PLUGINS_PACKAGE.replace('.', '/');
    private static final String BUILTIN_VRULES = PLUGINS_DIR + "builtin-vrules.ini";
    private static final String BUILTIN_PRT = PLUGINS_DIR + "builtin-protocols.ini";
    private static final String PLUGINS_PATH = "../plugins";
    private static final String PEAR_HOME = "/.pear";
    private static final String PLUGINS_HOME_PATH = PEAR_HOME + "/plugins";
    private static final String USING_JNLP_PROP = "usingJnlp";
    
    private static final Map configMap = new HashMap();
    
    private final JarClassLoader pluginClassLoader;
    private final boolean usingJNLP;
    
    private Configuration(String home)
    {
        usingJNLP = Boolean.getBoolean(USING_JNLP_PROP);
        String[] paths = { PLUGINS_PATH, home + PLUGINS_HOME_PATH };
        pluginClassLoader = new JarClassLoader(paths);
    }
    

// ----------------------------------------------------------------------------
    
    public static Configuration getCurrent()
    {
        String home = System.getProperty("user.home");
        Configuration c = (Configuration)configMap.get(home);
        if (c == null) configMap.put(home, c = new Configuration(home));
        return c;
    }
    
    public final String getHomeDir() { return System.getProperty("user.home") + PEAR_HOME; }
        
    public Class getPluginClass(String pluginName)
        throws ClassNotFoundException
    {
        return pluginClassLoader.loadClass(PLUGINS_PACKAGE + pluginName);
    }
    
    public static String getDefSourceName(String sourceName)
    {
        return PLUGINS_DIR + sourceName;
    }
    
    public Collection getBuiltinValidationRules()
        throws IOException
    {
        return this.getBuiltinResources(BUILTIN_VRULES, VR_EXT, false);
    }
    
    public Collection getBuiltinProtocols()
        throws IOException
    {
        return this.getBuiltinResources(BUILTIN_PRT, PRT_EXT, false);
    }

    public List getBuiltinResources(String confName, String ext, boolean merge)
        throws IOException
    {
        Collection c = null;
        InputStream in = null;
        
        if (confName != null)
        {
            in = pluginClassLoader.getResourceAsStream(confName);
            c = new HashSet(Arrays.asList(Configuration.parseBR(in)));
        }

        if (merge || c == null || c.isEmpty())
        {
            List l = pluginClassLoader.getResourceList(ext);
            if (merge && c != null) c.addAll(l);
            else c = l;
        }
        
        List res = new ArrayList(c);
        Collections.sort(res);
        return res;
    }
    
    public final InputStream getPluginResource(String name)
    {
        return pluginClassLoader.getResourceAsStream(PLUGINS_DIR + name);
    }
    
    public final InputStream getResourceAsStream(String name)
    {
        return pluginClassLoader.getResourceAsStream(name);
    }
    
    public final URL getResource(String name)
    {
        return pluginClassLoader.getResource(name);
    }
    
    public static final boolean isRuleSystem(String name)
    {
        return name.endsWith(VR_EXT);
    }
    
    public boolean isUsingJNLP()
    {
//        return pluginClassLoader.getResource("/pear.jnlp") != null;
        return usingJNLP; //Boolean.valueOf(System.getProperty(USING_JNLP_PROP, "false")).booleanValue();
    }
    
    public Collection getTabbers()
    {
        Collection res = null;
        
        try
        {
            /** @todo: caricamento dinamico tabbers */
            Class c = pluginClassLoader.loadClass(TABBER_PACKAGE + "xpe.XPEOutputTab");
            res = Arrays.asList(new TabPlugin[]{ (TabPlugin)c.newInstance() });
        }
        catch (Exception e) { e.printStackTrace(); }
        
        return res;
    }


// ----------------------------------------------------------------------------
    
    private static String[] parseBR(InputStream in)
        throws IOException
    {
        Reader rd = new BufferedReader(new InputStreamReader(in));
        StringWriter wr = new StringWriter();
        String[] res = NO_RESOURCES;
        
        try
        {
            while (true)
            {
                int ch = rd.read();
                if (ch == -1) break;
                wr.write(ch);
            }
            
            String src = wr.getBuffer().toString();
            res = src.split("\\s+");
            if (res.length == 1 && res[0].equals("")) res = NO_RESOURCES;
        }
        finally
        {
            try { rd.close(); } catch (Exception e) { e.printStackTrace(); }
            try { wr.close(); } catch (Exception e) { e.printStackTrace(); }
        }
        
        return res;
    }
}
