/*
 * ValidationState.java
 *
 * Created on 21 marzo 2004, 16.03
 */

package it.unive.dsi.pear.core;

import java.util.*;

import it.unive.dsi.pear.util.*;

/** This class is straightforward implementation of the interface
 * <CODE>it.unive.dsi.pear.BacktrackingHandler.State</CODE>
 * @author Francesco Placella
 */
public class ValidationState implements BackTrackingHandler.State
{
    private final Map stateMap = new HashMap();
    private final Map backupMap;
    private Object entity;
    private int brand;
    
    /** The sequential process iterator field. */    
    public static final Integer SEQ_PROC_ITR = new Integer(0);
    /** The instruction iterator field. */    
    public static final Integer INSTR_ITR = new Integer(1);
    /** The rule iterator field. */    
    public static final Integer RULE_ITR = new Integer(2);
    /** The auto pattern-match iterator field. */    
    public static final Integer PMATCH_ITR_MAP = new Integer(3);
    
    /** The container map field. */    
    public static final Integer CONTAINER_MAP = new Integer(4);
    /** The rule list field. */    
    public static final Integer RULE_LIST = new Integer(5);
    /** The substitution map field. */    
    public static final Integer SUBST_MAP = new Integer(6);
    /** The last backtraacking instruction field. */    
    public static final Integer BT_INSTRUCTION = new Integer(7);
    /** The last backtraacking rule field. */    
    public static final Integer BT_RULE = new Integer(8);

    /** Creates a new instance of ValidationState */    
    public ValidationState() { this(new HashMap()); }
    private ValidationState(final Map backupMap) { this.backupMap = backupMap; }
    
    public final void setFieldValue(Object field, CloneableElement value)
    {
        stateMap.put(field, value);
        backupMap.put(field, null);
    }
    
    public final CloneableElement getFieldValue(Object field)
    {
       return (CloneableElement)stateMap.get(field);
    }

    public CloneableElement getFieldValue(
        Object field, CloneableElement defaultValue)
    {
        CloneableElement res = this.getFieldValue(field);
        
        if (res == null)
        {
            this.setFieldValue(field, defaultValue);
            return defaultValue;
        }
        else
            return res;
    }

    public void backup(Object field)
    {
        CloneableElement val = (CloneableElement)this.stateMap.get(field);
        if (val != null) this.backupMap.put(field, val.clone());
    }

    /* clone creates a copy of the object using all the backup field values 
     * found instead of actual values.
     */
    public Object clone()
    {
        ValidationState vs = new ValidationState(this.backupMap);
        
        Iterator it = this.stateMap.entrySet().iterator();
        while (it.hasNext())
        {
            Map.Entry entry = (Map.Entry)it.next();
            Object key = entry.getKey();
            
            Object val = this.backupMap.get(key);
            if (val == null) val = entry.getValue();
            if (val != null)
                val = ((CloneableElement)val).clone();
            
            vs.stateMap.put(key, val);
        }
        
        vs.setBacktrackingEntity(this.getBacktrackingEntity());
        vs.setBrand(this.getBrand());
        
        return vs;
    }
    
    public Object getBacktrackingEntity() { return entity; }
    public void setBacktrackingEntity(Object entity) { this.entity = entity; }
    
    public CloneableElement retrieveFieldValue(Object field)
    {
        Object res = this.getFieldValue(field);
        this.setFieldValue(field, null);
        return (CloneableElement)res;
    }
    
    public int getBrand() { return brand; }
    public void setBrand(int brand) { this.brand = brand; }
    
    public String toString() { return stateMap.entrySet().toString(); }
    public Iterator fieldIterator() { return stateMap.keySet().iterator(); }
    
    public final void resetBackup() { backupMap.clear(); }
    
    public final BackTrackingHandler.State cloneState()
    {
        return (BackTrackingHandler.State)this.clone();
    }
    
}
