/*
 * ETBoolNode.java
 *
 * Created on 30 settembre 2004, 18.38
 */

package it.unive.dsi.pear.core.evaluationtree;

import java.util.*;

import it.unive.dsi.pear.core.parser.AndCheck;
import it.unive.dsi.pear.core.parser.Container;
import it.unive.dsi.pear.core.parser.BoolNode;
import it.unive.dsi.pear.core.parser.NotCheck;
import it.unive.dsi.pear.core.parser.Operation;
import it.unive.dsi.pear.core.parser.OrCheck;
import it.unive.dsi.pear.core.parser.ValidationRulesNode;

/**
 * This class implements the evaluation tree generic boolean node.
 * @author Francesco Placella
 */
public abstract class ETBoolNode extends ETNode
    implements Evaluable, BoolNode
{
    public ETBoolNode(ValidationRulesNode data) { super(data); } 
    
    protected Iterator dataIterator()
    {
        BoolNode bn = (BoolNode)this.getDataNode();
        return bn.boolIterator();
    }
    
    protected ETNode getChildInstance(ValidationRulesNode node)
    {
        if (node instanceof NotCheck) 
            return new ETNotCheck((NotCheck)node);
        else if (node instanceof AndCheck) 
            return new ETAndCheck((AndCheck)node);
        else if (node instanceof OrCheck) 
            return new ETOrCheck((OrCheck)node);
        else if (node instanceof Operation) 
            return new ETOperation((Operation)node);
        return null;
    }
    
// ----------------------------------------------------------------------------
    
    public boolean evaluate(EvalInfo evalInfo)
        throws Container.OperationException
    {
        this.buildPath(evalInfo);

        boolean value = true;
        boolean valid = false;
        int size = this.getChildCount();
        for (int i = 0; i < size && value; ++i)
        {
            ETBoolNode boolNode = (ETBoolNode)this.getChildAt(i);
            if (!boolNode.evaluable(evalInfo)) continue;
            value = value && (valid = boolNode.evaluate(evalInfo));
        }
        
        this.destroyPath(evalInfo);
        return valid;
    }
    
    protected final void buildPath(EvalInfo evalInfo)
    {
        evalInfo.getTreePath().add(this);
    }
    
    protected final void destroyPath(EvalInfo evalInfo)
    {
        evalInfo.getTreePath().remove(this);
    }
    
    /* evaluable checks if the current node has to be evaluated or has to be
     * skipped in order to restore the cporrect backtracking state.
     */
    protected boolean evaluable(EvalInfo evalInfo)
    {
        Operation btNode = (ETOperation)evalInfo.getBacktrackingEntity();
        boolean res = true;
        if (btNode != null && this.getClass() == ETOperation.class)
            res = btNode == this;
        return res;
    }
    
    public final int getBrand() { return this.hashCode(); }
    
    public final Iterator boolIterator()
    {
        return this.childrenIterator();
    }
}
