/*
 * ETCondition.java
 *
 * Created on 30 settembre 2004, 18.38
 */

package it.unive.dsi.pear.core.evaluationtree;

import java.util.*;

import it.unive.dsi.pear.util.*;

import it.unive.dsi.pear.core.parser.ConditionClause;
import it.unive.dsi.pear.core.parser.Condition;
import it.unive.dsi.pear.core.parser.Container;
import it.unive.dsi.pear.core.parser.Operations;
import it.unive.dsi.pear.core.parser.ValidationRulesNode;

/**
 * This class implements the evaluation tree condition node.
 * @author Francesco Placella
 */
public class ETCondition extends ETOpNode implements Condition
{
    private static int DEFAULT_BUF_SIZE = 30;
    
    public ETCondition(Condition data) { super(data); }
    
    protected final Condition getData()
    {
        return (Condition)this.getDataNode();
    }
    
    protected final Iterator dataIterator()
    {
        return this.getData().clauseIterator();
    }
    
    protected ETNode getChildInstance(ValidationRulesNode node)
    {
        return new ETConditionClause((ConditionClause)node);
    }

// ----------------------------------------------------------------------------
    
    public String toString()
    {
        StringBuffer buf = new StringBuffer(DEFAULT_BUF_SIZE);
        
        buf.append("[").append(this.getDefSrcType()).append("] ");
        
        Boolean b = this.getUniversalQuantification();
        if (b != null)
        {
            boolean v = b.booleanValue();
            buf.append(v ? "universally" : "existentially").
                append(" qualified ");
        }
        if ((b = this.getResultRequested()) != null)
            buf.append("(").append(b).append(") ");
        
        buf.append("condition");
        
        return buf.toString();
    }
    
    /* in this case evaluate implements the quantification logic, performimg
     * the appropriate backtracking steps if necessary.
     */    
    public boolean evaluate(EvalInfo evalInfo)
        throws Container.OperationException
    {
        this.buildPath(evalInfo);

        evalInfo.setResetAbstract(this.isResetAbstract());
        
        int size = this.getChildCount();
        boolean resultRequested = init(this.getResultRequested(), true);
        boolean universalQuantification =
            init(this.getUniversalQuantification(), size  > 1);
                
        BackTrackingHandler btHandler = evalInfo.getBtHandler();
        int superBrand = evalInfo.getNodeBrand();
        int brand = this.getBrand();
        evalInfo.setNodeBrand(brand);

        boolean value;
        boolean brandedAvailable;
        do
        {
            value = this.checkCondition(evalInfo, size) == resultRequested;
            brandedAvailable = false;            

            if (value && universalQuantification)
            {
                try
                {
                    brandedAvailable = btHandler.getBrandCount(brand) > 0;
                    if (brandedAvailable)
                    {
                        evalInfo.getEvaluator().backTrack();
                        BackTrackingHandler.State s = btHandler.getCurState();
                        Map m = 
                            (Map)s.getFieldValue(evalInfo.getSubstMapField());
                        evalInfo.setSubstMap(m);
                    }
                }
                catch (Exception e)
                {
                    throw new Container.OperationException(e);
                }
            }
        }
        while (value && brandedAvailable);
        
        if (universalQuantification)
        {
            if (!value) btHandler.removeBranded(superBrand);
            btHandler.removeBranded(brand);
        }
        
        evalInfo.setNodeBrand(superBrand);
        this.destroyPath(evalInfo);
        return value;
    }
    
    /* checkCondition actually evaluates the condition. */
    private boolean checkCondition(EvalInfo evalInfo, int size)
        throws Container.OperationException
    {
        boolean ipothesis = true;
        
        if (size > 1)
        {
            ETConditionClause bn = (ETConditionClause)this.getChildAt(0);
            ipothesis = !bn.evaluable(evalInfo) || bn.evaluate(evalInfo);
        }

        boolean thesis = true;
        if (ipothesis)
        {
            ETConditionClause bn = (ETConditionClause)this.getChildAt(size - 1);
            thesis = !bn.evaluable(evalInfo) || bn.evaluate(evalInfo);
        }

        return thesis;
    }
    
    private final boolean init(Boolean bool, boolean def)
    {
        return bool != null ? bool.booleanValue() : def;
    }
    
// ----------------------------------------------------------------------------
    
    /* the methods below gets their data from the related data tree. */
    
    public final Boolean getResultRequested()
    {
        return this.getData().getResultRequested();
    }
    
    public final Boolean getUniversalQuantification() {
        return this.getData().getUniversalQuantification();
    }
    
    public final boolean isResetAbstract() {
        return this.getData().isResetAbstract();
    }
    
    public final Iterator clauseIterator() { return this.childrenIterator(); }
}
