/*
 * ETNode.java
 *
 * Created on 30 settembre 2004, 17.35
 */

package it.unive.dsi.pear.core.evaluationtree;

import java.util.*;

import it.unive.dsi.pear.core.parser.ValidationRulesNode;
import it.unive.dsi.pear.util.DefaultTreeNode;

/**
 * This class implements the evaluation tree generic node.
 * @author Francesco Placella
 */
abstract public class ETNode extends DefaultTreeNode
    implements ValidationRulesNode
{
    private ValidationRulesNode dataNode;
    
    /** Builds an evaluation tree node given the corresponding data node. */    
    public ETNode(ValidationRulesNode data)
    {
        if (data != null) this.buildEvaluationTree(data);
    }
    
    /** Obtains a child instance from the given node. */
    abstract protected ETNode getChildInstance(ValidationRulesNode node);
    
    /** Returns an iterator over the data node children. */
    protected Iterator dataIterator()
    {
        return Collections.list(this.getDataNode().children()).iterator();
    }
    
    /** Builds the evaluation subtree following the shape of the given data 
     * subtree. */
    protected void buildEvaluationTree(ValidationRulesNode node)
    {
        this.setDataNode(node);

        int size = node.getChildCount();
        this.ensureCapacity(size);
        int index = 0;
        Iterator it = this.dataIterator();
        while (it.hasNext())
        {
            ValidationRulesNode child = (ValidationRulesNode)it.next();
            ETNode etc = this.getChildInstance(child);
            this.addChild(etc, index++);
        }
    }
    
    /** Getter for property dataNode.
     * @return Value of property dataNode.
     *
     */
    protected it.unive.dsi.pear.core.parser.ValidationRulesNode getDataNode() {
        return dataNode;
    }
    
    /** Setter for property dataNode.
     * @param dataNode New value of property dataNode.
     *
     */
    protected void setDataNode(ValidationRulesNode dataNode) {
        this.dataNode = dataNode;
    }
    
    /** Obtains the current node's instantiation appeared in the last 
     * evaluation process recorded. */
    public ETNode getLastInstantiation()
    {
        ETNode instance = (ETNode)this.cloneNode();
        
        Iterator it = this.childrenIterator();
        for (int i = 0; it.hasNext(); ++i)
        {
            ETNode child = (ETNode)it.next();
            instance.addChild(child.getLastInstantiation(), i);
        }
        
        return instance;
    }
}
