/*
 * ETRule.java
 *
 * Created on 30 settembre 2004, 18.38
 */

package it.unive.dsi.pear.core.evaluationtree;

import java.util.*;

import javax.swing.tree.TreeNode;

import it.unive.dsi.pear.core.parser.Conditions;
import it.unive.dsi.pear.core.parser.Instruction;
import it.unive.dsi.pear.core.parser.Operations;
import it.unive.dsi.pear.core.parser.Rule;
import it.unive.dsi.pear.core.parser.ValidationRulesNode;

/**
 * This class implements the evaluation tree rule node.
 * @author Francesco Placella
 */
public class ETRule extends ETNode implements Rule
{
    private static final int CHILD_COUNT = 2;
    
    public ETRule(Rule data) { super(data); }
    
    protected final Rule getData() { return (Rule)this.getDataNode(); }
    
    protected ETNode getChildInstance(ValidationRulesNode node) { return null; }
    
    protected void buildEvaluationTree(ValidationRulesNode node)
    {
        this.setDataNode(node);
        
        int index = 0;
        Rule rule = this.getData();
        Conditions c = rule.getConditions();
        if (c != null) this.addChild(new ETConditions(c), index++);
        Operations o = rule.getActions();
        if (o != null) this.addChild(new ETActions(o), index);
    }
    
// ----------------------------------------------------------------------------
    
    /* the methods below gets their data from the related data tree. */
    
    public String toString()
    {
        return this.getName() + "(" + this.getIdentity() + "): " + 
               this.getInstruction();
    }

    public final Conditions getConditions()
    {
        return (Conditions)this.getNode(ETConditions.class);
    }

    public final Operations getActions()
    {
        return (Operations)this.getNode(ETActions.class);
    }

    public final String getName() { return this.getData().getName(); }
    public final String getIdentity() { return this.getData().getIdentity(); }
    public final Instruction getInstruction() {  return this.getData().getInstruction(); }
    
    public boolean equals(Object obj)
    {
        if (obj == null) return false;
        return this.toString().equals(obj.toString());
    }
}
