/*
 * ETValidationRules.java
 *
 * Created on 30 settembre 2004, 18.38
 */

package it.unive.dsi.pear.core.evaluationtree;

import java.util.*;

import it.unive.dsi.pear.util.*;
import it.unive.dsi.pear.core.parser.Data;
import it.unive.dsi.pear.core.parser.Instruction;
import it.unive.dsi.pear.core.parser.Operation;
import it.unive.dsi.pear.core.parser.Rule;
import it.unive.dsi.pear.core.parser.SymbolTable;
import it.unive.dsi.pear.core.parser.TreeNode;
import it.unive.dsi.pear.core.parser.ValidationRules;
import it.unive.dsi.pear.core.parser.ValidationRulesNode;

/**
 * This class implements the evaluation tree root node.
 * @author Francesco Placella
 */
public class ETValidationRules extends ETNode
    implements EvaluationTree, ValidationRules
{
    public static final Rule[] EMPTY_ARRAY = {};
    
    private Map ruleMap = CloneableMap.EMPTY_MAP;
    
    public ETValidationRules(ValidationRules data) { super(data); }
    
    protected final ValidationRules getData()
    {
        return (ValidationRules)this.getDataNode();
    }
    
    protected final Iterator dataIterator()
    {
        return this.getData().ruleIterator();
    }
    
    protected ETNode getChildInstance(ValidationRulesNode node)
    {
        return new ETRule((Rule)node);
    }


// ----------------------------------------------------------------------------
    
    /* the methods below gets their data from the related data tree. */
    
    public String toString()
    {
        return "Validation Rules: " + this.getChildCount() + " definitions";
    }

    public final int getMaxTagNumber() { return this.getData().getMaxTagNumber(); }
    public final int getMinTagNumber() { return this.getData().getMinTagNumber(); }
    public final Map getContainerMap() { return this.getData().getContainerMap(); }
    public final String[][] getDefinedTags() {  return this.getData().getDefinedTags(); }
    public final SymbolTable getSymbolTable() { return this.getData().getSymbolTable(); }
    public final Map getTagTypes() { return this.getData().getTagTypes(); }
    public final String getDescription() { return this.getData().getDescription(); }
    
    public final Rule[] getMatchingRules(final String instrName)
    {
        Collection c = (Collection)this.getRuleMap().get(instrName);
        return c != null ? (Rule[])c.toArray(EMPTY_ARRAY) : EMPTY_ARRAY;
    }

    /* getRuleMap build a map indexed by the name of the instructions
     * returning a list of the rules defined on the related instruction.
     */
    private Map getRuleMap()
    {
        if (ruleMap.equals(CloneableMap.EMPTY_MAP))
        {
            int size = this.getChildCount();
            ruleMap = new HashMap(size);
            
            for (int i = 0; i < size; ++i)
            {
                TreeNode node = (TreeNode)this.getChildAt(i);
                if (node instanceof Rule)
                {
                    Instruction ruleInstr = ((Rule)node).getInstruction();
                    String name = ruleInstr.getName();
                    LinkedList ruleList = (LinkedList)ruleMap.get(name);
                    if (ruleList == null)
                    {
                        ruleList = new LinkedList();
                        ruleMap.put(name, ruleList);
                    }
                    ruleList.add(node);
                }
            }
        }
        
        return ruleMap;
    }
    
    public final Iterator ruleIterator() { return this.childrenIterator(); }
}
