/*
 * AbstractData.java
 *
 * Created on 28 marzo 2004, 23.26
 */

package it.unive.dsi.pear.core.parser;

import java.util.*;

/** This class implements the behaviour of the <CODE>...</CODE> construct.
 * @author Francesco Placella
 */
public class AbstractData extends SimpleData
{
    public static final AbstractData ABSTRACT_DATA = new AbstractData();
    
    public static final String ABSTRACT_INFO = "...";
    public static final String EMPTY = "";
    
    /** Creates a new instance of <CODE>AbstractData</CODE>.
     * @param info the string representation of the construct
     */
    protected AbstractData(String info) { super(); super.setInfo(info); }
    /** Creates a new instance of <CODE>AbstractData</CODE>. */    
    public AbstractData() { this(ABSTRACT_INFO); }
    public void setInfo(String info) {}
    /** Allows subclasses to change the string representation of the construct.
     * @param ad the new string representation
     */    
    protected void narrowInfo(AbstractData ad) { super.setInfo(ad.getInfo()); }
    
    public boolean putMatching(Map substMap, List iList)
    {
        Iterator it = iList.iterator();
        while (it.hasNext())
        {
            Object obj = it.next();
            if (obj instanceof TaggedData) return false;
            if (obj == null) it.remove();
        }
        
        return this.matchAbstract(substMap, iList);
    }
    
    /** Adds to the given substitution map a mapping for the given data list.
     * @param substMap the substitution map
     * @param iList the list of data elements matched by the construct
     * @return <CODE>true</CODE>
     */    
    public boolean matchAbstract(Map substMap, List iList)
    {
        if (!iList.isEmpty()) substMap.put(this.getInfo(), iList);
        return true;
    }
    
    /** Adds the element matched by the construct in the given substitution map
     * to the list passed as argument.
     * @param substMap the substitution map
     * @param contentList the substitution list
     * @return <CODE>true</CODE> if there is no matching for the construct,
     * <CODE>false</CODE> otherwise
     */    
    public boolean subst(Map substMap, List contentList)
    {
        Object obj = substMap.get(this.getInfo());
        if (EMPTY.equals(obj) || obj == null) return true;

        List list = (List)obj;
        Iterator it = list.iterator();
        while (it.hasNext())
        {
            Data d = (Data)it.next();
            contentList.add(d);
            d.setMatched(true);        
        }
        
        return false;
    }
}
