/*
 * AsymKeyData.java
 *
 * Created on 15 giugno 2004, 11.19
 */

package it.unive.dsi.pear.core.parser;

import java.util.*;

/** This class models an asymmetric key declaration.
 * @author Francesco Placella
 */
public class AsymKeyData extends KeyData
{
    public static final String KEY_INSTR = "asym-key";
    
    /** Creates a new instance of <CODE>AsymKeyData</CODE>.
     * @param key the key name
     */
    public AsymKeyData(String key) { super(key); }
    /** Creates a new instance of <CODE>AsymKeyData</CODE>.
     * @param key the key name
     * @param principal the principal owning the key pair
     */    
    public AsymKeyData(String key, SimpleData[] principal)
    {
        super(key, null);
        this.setPrincipals(new SimpleData[]{ principal[0] });
    }

    /** this class is a straightforward implementation of 
     * <CODE>KeyInstance</CODE>.
     */
    public static class KeyInstance extends SimpleData
        implements it.unive.dsi.pear.core.parser.KeyInstance
    {
        private boolean pub;
        /** Creates a new instance of <CODE>KeyInstance</CODE>.
         * @param info the key name
         * @param pub the part to be instantiated
         */        
        public KeyInstance(String info, boolean pub)
        {
            super(info);
            this.pub = pub;
        }
        
        public boolean isPublic() { return pub; }
        public int hashCode() { return this.toString().hashCode(); }
        public Data getKey() { return new SimpleData(this.getInfo()); }
        
        public boolean putMatching(Map substMap, List iList)
        {
            Class rClass = this.getClass();
            ListIterator it = iList.listIterator();
            while (it.hasNext())
            {
                Object obj = it.next();
                if (obj != null && rClass == obj.getClass())
                {
                    KeyInstance key = (KeyInstance)obj;
                    List list = Arrays.asList(new Data[]{ key });
                    if (this.isPublic() == key.isPublic() &&
                        super.putMatching(substMap, list))
                    {
                        it.set(null);
                        return true;
                    }
                }
            }
            
            return false;        
        }
        
        public String toString()
        {
            return (pub ? "Pub" : "Priv") + "(" + super.toString() + ")" ;
        }
        
        public KeyType getKeyType()
        {
            return pub ? KeyType.PUB_KEY : KeyType.PRIV_KEY;
        }
    }
    
    protected String keyInstr() { return KEY_INSTR; }
    
    public void setPrincipals(Data[] p)
    {
        super.forceContent(new Data[]{ new InstructionImpl(KEY_INSTR, p) });
    }
    
    /** Returns the specified key instance.
     * @param pub if <CODE>true</CODE> it's returned the public part,
     * otherwise the private one
     */    
    public KeyInstance getInstance(boolean pub)
    {
        return new KeyInstance(this.getInfo(), pub);
    }
    
    public KeyType getKeyType() { return KeyType.KEY_PAIR; }
}
