/*
 * CipherInstructionImpl.java
 *
 * Created on 27 agosto 2004, 0.51
 */

package it.unive.dsi.pear.core.parser;

/** This class is an abstract implementation of <CODE>CipherInstruction</CODE>.
 * @author Francesco Placella
 */
public abstract class CipherInstructionImpl extends InstructionImpl
    implements CipherInstruction
{
    protected static final int DEFAULT_BUF_SIZE = 30;
    
    /** Creates a new instance of <CODE>CipherInstructionImpl</CODE>.
     * @param name the name of the instruction
     * @param cd the ciphertext argument
     */
    public CipherInstructionImpl(String name, Ciphertext cd)
    {
        super(name, new Data[]{ cd });
    }    

    public String getCiphertextImage()
    {
        Ciphertext cd = this.getCiphertext();
        String info = cd.getInfo();
        boolean encrypted = cd.isEncrypted();
        cd.setInfo(null);
        cd.setEncrypted(true);
        String res = cd.toString();
        cd.setInfo(info);
        cd.setEncrypted(encrypted);
        return res;
    }

    public final Ciphertext getCiphertext()
    {
        return (CompositeData)this.getData()[0];
    }
    
    public boolean isCompatible(CipherInstruction ci)
    {
        if (this.getClass() != ci.getComplementClass()) return false;
        
        Ciphertext td = this.getCiphertext();
        Ciphertext cd = ci.getCiphertext();
        
        Data key = td.getKey();
        Data cKey = cd.getKey();
        
        if (key.getClass() != cKey.getClass()) return false;
        if (key instanceof AsymKeyData.KeyInstance)
        {
            AsymKeyData.KeyInstance kt = (AsymKeyData.KeyInstance)key;
            AsymKeyData.KeyInstance kc = (AsymKeyData.KeyInstance)cKey;
            if (kt.isPublic() == kc.isPublic()) return false;
        }
        
        Data[] content = td.getContent();
        Data[] ciContent = cd.getContent();

        int size = content.length;
        if (size != ciContent.length) return false;
        
        for (int i = 0; i < size; ++i)
            if (content[i].getClass() != ciContent[i].getClass())
                return false;
        
        return true;
    }
    
    public boolean isEncrypt()
    {
        return this instanceof Encrypt;
    }
    
}
