/*
 * Ciphertext.java
 *
 * Created on 27 settembre 2004, 16.39
 */

package it.unive.dsi.pear.core.parser;

import java.util.*;

/** This interface models a ciphertext: it may have an identifier associated,
 * it must have a key (symmetric or asymmetric) and a content.
 * @author Francesco Placella
 */
public interface Ciphertext extends Data
{
    /** Checks if the given key has the same type of the one associated with the
     * current ciphertext.
     * @param sd the key to be matched
     */    
    public boolean keyMatches(Data sd);
    /** Returns <CODE>true</CODE> if the ciphertext contains abstract data,
     * <CODE>false</CODE> otherwise.
     */    
    public boolean isAbstract();
    /** Returns <CODE>true</CODE> if the key associated to the ciphertext is
     * asymmetric, <CODE>false</CODE> otherwise.
     */    
    public boolean isAsymmetric();
    /** Returns an array containing the ciphertext components. */    
    public Data[] getContent();
    /** Sets the new value for the ciphertext content. */    
    public void setContent(Data[] content);
    /** Returns <CODE>true</CODE> if the ciphertext is encrypted with the 
     * associated key, <CODE>false</CODE> if it is decrypted.
     */    
    public boolean isEncrypted();
    /** If the given argument is <CODE>true</CODE> the ciphertext is encrypted,
     * otherwise decrypted, with the associated key.
     */    
    public void setEncrypted(boolean encrypted);
    /** Returns a data element representing the related key. */    
    public Data getKey();
    /** Returns the set of tags used in the ciphertext content. */    
    public Collection getUsedTags();   
}