/*
 * Data.java
 *
 * Created on 7 marzo 2004, 13.04
 */

package it.unive.dsi.pear.core.parser;

import java.util.*;

import it.unive.dsi.pear.util.*;

/** This interface models a generic data element that has an associated
 * identifier named <CODE>info</CODE>.
 * @author Francesco Placella
 */
public interface Data extends Container
{
    public static final Integer UNIQUE_IDS = new Integer(0);
    public static final Data[] EMPTY_DATA = new Data[0];
    
    /** Returns the type of the key associated with the data element if any. */    
    public KeyType getKeyType();
    /** Returns the name of the identifer associated with the data element. */    
    public String getInfo();
    /** Sets a new value for the name of the identifier associated with the data
     * element. */    
    public void setInfo(String name);
    /** Changes the data element status to matched or unmatched according to the
     * value of the given argument. */    
    public void setMatched(boolean matched);
    /** Checks if the data element is already matched. */    
    public boolean isMatched();
    /** Checks if the data element is a simple identifier or something more
     * elaborate. */
    public boolean isSimple();
    /** Returns the number of tagged elements contained in the current data
     * element, if any.
     */
    public int getTagNumber();
    
    /** Performs a pattern-matching between the current data element and the 
     * given one: if successful the current element's components are substituted 
     * with the values defined in the substitution map.
     * @param data the data element to be matched
     * @param substMap the substitution map: it may already contain some 
     * mappings
     * @return <CODE>true</CODE> if the pattern-match is successful,
     * <CODE>false</CODE> otherwise.
     */    
    public boolean autoSubst(Data data, Map substMap);
    /** Creates a copy of the current data element. */    
    public Object clone();
    /** Checks if the current data element contains tagged ones. */    
    public boolean containsTagged();
    /** Create a new instance of the simple data element.
     * @param info the name of the associated identifier.
     */    
    public Data newData(String info);
    /** Performs a pattern-matching between the current data-element and one of
     * the elements contained in the given list.
     * @param substMap the substitution map is populated with the mappings 
     * needed to match the selected element in the list; it may already contain 
     * some mappings.
     * @param iList a list of candidates to the matching: the first succesfully 
     * matched is choosen
     * @return <CODE>true</CODE> if the matching is successful,
     * <CODE>false</CODE> otherwise.
     */    
    public boolean putMatching(Map substMap, List iList);    
    /** Replaces the component found in the given substitution map with their
     * mapped values.
     * @param substMap the substitution map
     */
    public void subst(Map substMap);
    /** Replaces the component found in the given substitution map with their
     * mapped values.
     * @param removeAbstract if <CODE>true</CODE> any mapping for the abstract 
     * data is removed from the substitution map
     * @param substMap the substitution map
     */
    public void subst(Map substMap, boolean removeAbstract);
    /** Creates a new instance of a tagged element.
     * @param id the identifier to be tagged
     * @param tag the tag to be associated to the identifier
     */    
    public TaggedData tag(String id, String tag);
}
