/*
 * LogError.java
 *
 * Created on 27 settembre 2004, 12.16
 */

package it.unive.dsi.pear.core.parser;

import it.unive.dsi.pear.log.Log;

/** This class implements a log error: it builds a <CODE>Log.Message</CODE>
 * from a <CODE>ParseException</CODE>.
 * @author Francesco Placella
 */
public class LogError implements Log.Message
{
    private final int[] coords;
    private final String message;
    private final String source;

    public LogError(ParseException pe)
    {
        message = pe.getMessage().replaceAll("\r\n *", " ");
        GenericToken token = pe.currentToken.getNext();
        coords = new int[] { token.getBeginLine(), token.getBeginColumn() };
        source = token.toString();                
    }

    public int getLine() { return coords[0]; }
    public int getColumn() { return coords[1]; }
    public String getMessage() { return message; }
    public String getSource() { return source; }

    public String toString()
    {
        return message + " at " + coords[0] + ":" + coords[1];
    }    
}
