/*
 * MultiTaggedDataImpl.java
 *
 * Created on 17 gennaio 2005, 13.23
 */

package it.unive.dsi.pear.core.parser;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;

/**
 *
 * @author Francesco Placella
 */
public class MultiTaggedDataImpl extends ParametricTaggedDataImpl
    implements MultiTaggedData
{
    private static final int DEFAULT_BUF_SIZE = 30;
    private static final Integer ABSTRACT_ID = new Integer(-1);
    
    private Data[] multi;
    private LabeledAbstractData abstractMulti;
    private String abstractLabel;
    
    /** Creates a new instance of MultiTaggedDataImpl */
    public MultiTaggedDataImpl(Data[] multi, String tag)
    {
        super();
        this.setTag(tag);
        this.multi = multi;
        
        if (this.isAbstract()) 
        {
            abstractMulti = new LabeledAbstractData(this.getAbstractInfo());
            abstractLabel = abstractMulti.getInfo();
        }
    }
    
    public String toString()
    {
        StringBuffer buf = new StringBuffer(DEFAULT_BUF_SIZE);
        
        String info = this.getInfo();
//        if (info == null)
        {
            buf.append("{"); 
            this.appendData(multi, buf).append("}");
        }
//        else buf.append(info); 
        
        buf.append(":").append(this.getTag());
        
        Data[] p = this.getParameters();
        if (p != null)
        {
            buf.append("(");
            this.appendData(p, buf);
            buf.append(")");
        }
        
        return buf.toString();
    }
    
    public boolean putMatching(Map substMap, List iList)
    {
        Class rClass = this.getClass();
        ListIterator it = iList.listIterator();
        while (it.hasNext())
        {
            Object obj = it.next();
            if (obj == null) continue;
            if (rClass == obj.getClass())
            {
                MultiTaggedDataImpl iData = (MultiTaggedDataImpl)obj;
                if (this.getTag().equals(iData.getTag()) &&
                    this.matchParameters(substMap, iData))
                {
                    Data[] iMulti = iData.getMulti();

                    // if multis have the same length, they are treated as not abstract
                    if (this.isAbstract() && !iData.isAbstract())
                    {
                        if (substMap.get(abstractLabel) != null) return false;
                        abstractMulti.putMatching(substMap, Arrays.asList(iMulti));
                    }
                    else
                    {
                        Data[] multi = this.getMulti();
                        int size = iMulti.length;
                        if (size != multi.length) return false;
                        for (int i = 0; i < size; ++i)
                        {
                            List ml = Arrays.asList(new Data[]{ iMulti[i] });
                            if (!multi[i].putMatching(substMap, ml))
                                return false;
                        }
                    }
                    
                    it.set(null);
                    return true;
                }
            }
            else if (obj instanceof TaggedData && this.isAbstract())
            {
                TaggedData td = new TaggedDataImpl(abstractLabel, this.getTag());
                List tdList = Arrays.asList(new Data[]{ (Data)obj });
                if (td.putMatching(substMap, tdList))
                {
                    it.set(null);
                    return true;
                }
            }
        }
        return false;
    }
    
    public void subst(Map substMap, boolean removeAbstract)
    {
//        super.subst(substMap, removeAbstract);
        
        if (this.isAbstract())
        {
            Object obj = substMap.get(abstractLabel);
            if (obj == null)
            {
                obj = substMap.get(this.getAbstractInfo());
                if (obj == null) return;
            }

            if (obj instanceof Data)
            {
                String info = ((Data)obj).getInfo();
                super.setInfo(info);
                multi[0].setInfo(info);
            }
            else
            {
                List l = (List)obj;
                multi = (Data[])l.toArray(Data.EMPTY_DATA);
            }
        }
        else
        {
            for (int i = 0; i < multi.length; ++i)
                multi[i].subst(substMap, removeAbstract);
        }
    }
    
    public Object clone()
    {
        MultiTaggedDataImpl cloned = (MultiTaggedDataImpl)super.clone();
        
        Data[] cMulti = cloned.getMulti();
        cloned.multi = (Data[])cMulti.clone();
        
        int size = cMulti.length;
        for (int i = 0; i < size; ++i)
        {
            Data data = cMulti[i];
            if (data != null) cloned.multi[i] = (Data)data.clone();
        }
        
        return cloned;
    }
    
    public boolean equals(Object obj)
    {
        if (obj == null || !(obj instanceof MultiTaggedData)) return false;
        
        MultiTaggedData mtd = (MultiTaggedData)obj;
        if (!this.getTag().equals(mtd.getTag())) return false;
        
        Data[] iMulti = mtd.getMulti();
        for (int i = 0; i < multi.length; ++i)
            if (!multi[i].equals(iMulti[i])) return false;
        
        return true;
    }

    public int hashCode()
    {
        int hashCode = this.getTag().hashCode();
        for (int i = 0; i < multi.length; ++i) hashCode += multi[i].hashCode();
        return hashCode;
    }
    
    public final Object[] toArray() { return this.getMulti(); }
    public Data[] getMulti() { return multi; }
    public void setInfo() {}
    
    private final boolean isAbstract() { return multi.length == 1; }
    private final String getAbstractInfo() { return multi[0].getInfo(); }
}
