/*
 * ParametricTaggedDataImpl.java
 *
 * Created on 16 giugno 2004, 16.53
 */

package it.unive.dsi.pear.core.parser;

import java.util.*;

/** This class implements a data element with parametric tag.
 * @author Francesco Placella
 */
public class ParametricTaggedDataImpl extends TaggedDataImpl
    implements ParametricTaggedData
{
    private static final int DEFAULT_BUF_SIZE = 20;
    
    private Data[] parameters;    
    
    /** Creates a new instance of <CODE>ParametricTaggedDataImpl</CODE>. */
    public ParametricTaggedDataImpl() {}
    /** Creates a new instance of <CODE>ParametricTaggedDataImpl</CODE>.
     * @param info the data element name
     * @param tag the tag name
     * @param parameters the tag parameters
     */    
    public ParametricTaggedDataImpl(String info, String tag, Data[] parameters)
    {
        super(info, tag);
        this.parameters = parameters;        
    }
    
    public boolean equals(Object obj)
    {
        if (!(obj instanceof ParametricTaggedData) || !super.equals(obj))
            return false;
        
        ParametricTaggedDataImpl ptd = (ParametricTaggedDataImpl)obj;
        int size = parameters.length;
        for (int i = 0; i < size; ++i)
            if (!parameters[i].equals(ptd.parameters[i])) return false;
        
        return true;
    }
    
    public String toString()
    {
        StringBuffer buf = new StringBuffer(DEFAULT_BUF_SIZE);
        buf.append(super.toString()).append("(");
        this.appendData(parameters, buf).append(")");
        return buf.toString();
    }
    
    public boolean putMatching(Map substMap, List iList)
    {
        Class rClass = this.getClass();
        ListIterator it = iList.listIterator();
        while (it.hasNext())
        {
            Object obj = it.next();
            if (obj != null && rClass == obj.getClass())
            {
                ParametricTaggedData iData = (ParametricTaggedData)obj;
                List tdList = Arrays.asList(new Data[]{ iData });
                if (super.putMatching(substMap, tdList))
                {
                    if (!this.matchParameters(substMap, iData)) return false;
                    it.set(null);
                    return true;
                }
            }
        }
        return false;
    }
    
    /** Getter for property parameters.
     * @return Value of property parameters.
     *
     */
    public Data[] getParameters() {
        return this.parameters;
    }
    
    /** Setter for property parameters.
     * @param parameters New value of property parameters.
     *
     */
    public void setParameters(Data[] parameters) {
        this.parameters = parameters;
    }
    
    protected boolean matchParameters(Map substMap, ParametricTaggedData iData)
    {
        Data[] iParameters = iData.getParameters();
        Data[] parameters = this.getParameters();
        
        if (iParameters != null &&  parameters != null)
        {
            int size = iParameters.length;
            if (size != parameters.length) return false;
            for (int i = 0; i < size; ++i)
            {
                List par = Arrays.asList(new Data[]{ iParameters[i] });
                if (!parameters[i].putMatching(substMap, par))
                    return false;
            }
        }
        else if (parameters != iParameters) return false;
        
        return true;
    }
    
    protected StringBuffer appendData(Data[] data, StringBuffer buf)
    {
        if (data != null)
        {
            int size = data.length;        
            if (size > 0)
            {
                buf.append(data[0]);            
                for (int i = 1; i < size; ++i)
                    buf.append(", ").append(data[i]);
            }
        }
        
        return buf;
    }
}
