/*
 * Protocol.java
 *
 * Created on 27 settembre 2004, 11.16
 */

package it.unive.dsi.pear.core.parser;

import java.util.*;

import it.unive.dsi.pear.util.*;

/** This interface models a generic protocol encoding.
 * @author Francesco Placella
 */
public interface Protocol extends ProtocolNode, CloneableElement
{
    /** Returns a <CODE>Comparator</CODE> that can be used to compare the
     * sequential processes by their statistics.
     */    
    public Comparator getStatisticsComparator();
    /** Returns an iterator over the child sequential processes. */    
    public CloneableIterator seqProcIterator();
    /** Returns an iterator over the child key bindings. */
    public Iterator keyBindingIterator();
    /** Returns an iterator over the child sequnetial process calls. */
    public Iterator callIterator();
    
    /** Reorder the definitions of the sequential processes according to the
     * order specified.
     * @param sps an array containing the sequential processes reordered
     */    
    public void reorderSeqProcesses(SequentialProcess[] sps);
    /** Returns a string image of the protocol source. */    
    public String getProtocolImage();
    /** Returns the protocol name. */    
    public String getProtocolName();
    /** Returns the taggings associated to the current protocol. */    
    public Protocol[] getTaggings();
    /** Sets the taggings for the current protocol. */    
    public void setTaggings(Protocol[] taggings);
    /** Checks if the current protocol is a tagging. */    
    public boolean isTagging();
    /** Marks or unmarks this protocol as a tagging. */    
    public void setTagging(boolean tagging);
    /** Returns the names of the rule systems associated with the protocol */
    public Collection getAssociatedRules();
    /** Returns a protocol's description */
    public String getDescription();
}
