/*
 * RootData.java
 *
 * Created on 17 aprile 2004, 16.19
 */

package it.unive.dsi.pear.core.parser;

import java.util.*;

import it.unive.dsi.pear.util.*;

/** This class is an abstract implementation of <CODE>Data</CODE>.
 * @author Francesco Placella
 */
public abstract class RootData implements Data
{
    protected static final int ABSTRACT_INDEX = -1;
    
    private boolean matched = false;
    
    // viewed as a Container a RootData object is read-only
    public final void clear() {}
    public final boolean add(Data data) { return false; }
    public final boolean add(Data key, Data data) { return false; }
    public final boolean delete(Data data) { return false; }
    public final String getContentImage() { return this.content().toString(); }
    
    public Object[] toArray() { return new Data[]{ this }; }
    public final Collection content() { return Arrays.asList(this.toArray()); }
    
    public CloneableIterator infoIterator()
    {
        return new ArrayIterator(this.content().toArray());
    }
    
    public boolean autoPatternMatch(Data arg, Data data, Map substMap)
    {
        return arg.autoSubst(data, substMap);
    }
    
    public int hashCode() { return this.rootHashCode(); }
    /** Computes an hash code: it may be used by all the element of the
     * <CODE>Data</CODE> hierarchy.
     */    
    protected int rootHashCode()
    {
        int hashCode = 0;
        
        String info = this.getInfo();
        if (info != null) hashCode += info.hashCode();
        
        Iterator it = this.content().iterator();
        while (it.hasNext())
        {
            Data d = (Data)it.next();
            if (d != null && d != this) hashCode += d.hashCode();
        }

        return hashCode != 0 ? hashCode : super.hashCode();
    }
    
    public Object clone()
    {
        try { return super.clone(); }
        catch (CloneNotSupportedException cns) { return null; }
    }
    
    public boolean containsTagged() { return false; }
    public int getTagNumber() { return 0; }
    
    public Data get(Data d)
    {
        String info = d.getInfo();
        Iterator it = this.content().iterator();
        while (it.hasNext())
        {
            Data data = (Data)it.next();
            String dInfo = data.getInfo();
            if (dInfo != null && dInfo.equals(info)) return data;
        }

        return null;
    }
    
    public abstract void subst(Map substMap, boolean removeAbstract);
    public final void subst(Map substMap) { this.subst(substMap, true); }
    
    public boolean autoSubst(Data data, Map substMap)
    {
        List iData = Arrays.asList(new Data[]{ data });
        boolean res = this.putMatching(substMap, iData);

        if (res)
        {
            Object obj = substMap.get(AbstractData.ABSTRACT_INFO);
            if (obj == null)
            {
                substMap.put(AbstractData.ABSTRACT_INFO, AbstractData.EMPTY);
                this.subst(substMap);
                substMap.remove(AbstractData.ABSTRACT_INFO);
            }
            else this.subst(substMap);
        }
        return res;
    }
    
    public boolean isMatched() { return matched; }
    public void setMatched(boolean matched) { this.matched = matched; }
    
    public KeyType getKeyType() { return KeyType.NO_KEY; }
    
    public boolean isSimple() { return this.getClass() == SimpleData.class; }
    
    public final TaggedData tag(String id, String tag)
    {
        return new TaggedDataImpl(id, tag);
    }
    
    public final Data newData(String info) { return new SimpleData(info); }
}