/*
 * SequentialProcess.java
 *
 * Created on 27 settembre 2004, 11.40
 */

package it.unive.dsi.pear.core.parser;

import java.util.*;

import it.unive.dsi.pear.util.*;

/** This interface models a generic sequential process.
 * @author Francesco Placella
 */
public interface SequentialProcess extends ProtocolNode
{
    /** Returns the pseudo instruction corresponding to the given data element.
     * @param arg a signature data element     
     */    
    public Instruction getDeclaration(Data arg);
    /** Returns an iterator over the child instructions. */    
    public CloneableIterator instrIterator();
    /** Returns an iterator over the signature declarations. */    
    public Iterator declIterator();
    /** Return the sequential process signature. */    
    public Instruction getDeclaration();

    /** Returns the principal assigned to the sequential process. */    
    public Data getSelf();
    /** Returns the set of identifiers that may hae only one matching in a
     * substitution map (the identities). */    
    public Set getUniqueIds();
    /** Returns the parent protocol. */    
    public Protocol getProtocol();
    /** Returns the statistics computed for this sequential process. */    
    public int getStatistics();
    /** Sets the statistics computed for the current sequential process. */    
    public void setStatistics(int statistics);
    /** Returns the symbol table associated to the current sequential process.
     */    
    public Map getSymbolTable();
}
