/*
 * SimpleData.java
 *
 * Created on 20 febbraio 2004, 16.54
 */

package it.unive.dsi.pear.core.parser;

import java.util.*;

/** This is a straightforward implementation of the <CODE>Data</CODE>
 * interface: it models a simple identifier.
 * @author Francesco Placella
 */
public class SimpleData extends RootData
{
    public static final String ANY_VAL = "any";
    
    private String info;
    private boolean isAny = false;
    
    /** Holds value of property matchable. */
    private boolean matchable = true;
    
    /** Creates a new instance of SimpleData. */
    public SimpleData() {}
    /** Creates a new instance of SimpleData.
     * @param info the name of the identifier
     */    
    public SimpleData(String info) { this.setInfo(info); }
    
    public String toString() { return info; }
    
    public int hashCode() { return this.toString().hashCode(); }
    
    public boolean putMatching(Map substMap, List iList)
    {
        if (!this.isMatchable()) return false;
        
        String info = this.getInfo();
        boolean thisNull = info == null;
        
        boolean matched = this.isMatched();
        Class rClass = this.getClass();
        ListIterator it = iList.listIterator();
        while (it.hasNext())
        {
            Data iData = (Data)it.next();
            if (iData != null)
            {
                if (rClass == iData.getClass())
                {
                    if (matched)
                        if (this.equals(iData))
                        {
                            it.set(null);
                            return true;
                        }
                        else continue;

                    String iInfo = iData.getInfo();
                    if (thisNull && iInfo == null) return true;

                    if (rClass != SimpleData.class) 
                        iData = new SimpleData(iInfo);
                    else
                        if (!((SimpleData)iData).isMatchable()) continue;

                    if (!isAny)
                    {
                        Data data = (Data)substMap.get(info);                
                        if (data == null || data == AbstractData.ABSTRACT_DATA)
                        {
                            Set uids = (Set)substMap.get(Data.UNIQUE_IDS);
                            if (uids != null && uids.contains(iData.getInfo()) &&
                                substMap.containsValue(iData)) continue;

                            substMap.put(info, iData);
                        }
                        else if (!data.equals(iData)) return false;
                    }
                    else substMap.put(info, iData);

                    it.set(null);
                    return true;
                }
            }
        }
        return false;
    }

    /** Getter for property info.
     * @return Value of property info.
     *
     */
    public final String getName() { return this.getInfo(); }
    public String getInfo() { return info; }
    
    /** Setter for property info.
     * @param info New value of property info.
     *
     */
    public final void setName(String info) { this.setInfo(info); }
    public void setInfo(String info)
    {
        isAny = ANY_VAL.equals(info);
        this.info = info;
    }
    
    public boolean contains(Data data) { return this.equals((Object)data); }
    
    public boolean equals(Object obj)
    {
        if (obj == null) return false;
        if (!(obj instanceof SimpleData)) return false;
        
        SimpleData sd = (SimpleData)obj; 
        String info = sd.getInfo();
        String thisInfo = this.getInfo();

        if (thisInfo == null || info == null) return thisInfo == info;
        
        return isAny || sd.isAny || thisInfo.equals(info);
    }
    
    public void subst(Map substMap, boolean removeAbstract)
    {
        if (!this.isMatched())
        {
            Data subst = (Data)substMap.get(this.getInfo());
            if (subst != null)
            {
                this.setInfo(subst.getInfo());
                this.setMatched(true);
            }
        }
    }

    public KeyType getKeyType() { return KeyType.SYM_KEY; }
    
    // an unmatchable element can't appear in a pattern-matching process
    
    /** Getter for property matchable.
     * @return Value of property matchable.
     *
     */
    public boolean isMatchable() {
        return this.matchable;
    }
    
    /** Setter for property matchable.
     * @param matchable New value of property matchable.
     *
     */
    public void setMatchable(boolean matchable) {
        this.matchable = matchable;
    }
}
