/*
 * SymbolTable.java
 *
 * Created on 30 settembre 2004, 23.12
 */

package it.unive.dsi.pear.core.parser;

/** This interface models a symbol table.
 * @author Francesco Placella
 */
public interface SymbolTable extends java.util.Map
{
    /** The given token is assigned the given type and recorded in the current 
     * token list.
     * @param tok the token to be recorded
     * @param type the type to be assigned
     * @throws SemanticException if the token is already declared
     * @return the given type
     */    
    public EnumType putListElement(final GenericToken tok, final EnumType type)
        throws SemanticException;
    /** The given string is treated as a token: it is assigned the given type 
     * and recorded in the current token list.
     */    
    public EnumType putListElement(final String symbol, final EnumType type)
        throws SemanticException;
    /** The given data element is treated as a token: it is assigned the given
     * type and recorded in the current token list.
     */    
    public EnumType putListElement(final Data symbol, final EnumType type)
        throws SemanticException;

    /** Checks if the given symbol has the specified type assigned: if so the 
     * symbol is added to the current token list, otherwise the list is cleared 
     * and a semantic exception is raised.
     * @param symbol the symbol to be checked
     * @param type the type to be checked
     * @throws SemanticException if the check fails
     */    
    public void checkListElement(final String symbol, final EnumType type)
        throws SemanticException;

    /** Checks if the given symbol is already declared.
     * @param symbol the symbol to be checked
     * @throws SemanticException if the check fails
     * @return the given symbol
     */    
    public String checkDeclaration(final String symbol)
        throws SemanticException;
    
    /** Checks if the specified symbol has the given type.
     * @throws SemanticException if the check fails, reporting the expected type
     * if any
     * @return the give type
     */    
    public String checkType(final String symbol, final EnumType type)
        throws SemanticException;
    /** Checks if the specified symbol has one of the given types.
     * @param symbol the symbol to be checked
     * @param type an array containing the expected types
     * @throws SemanticException if the check fails, reporting the expected 
     * type if any
     * @return the type actually assigned to the symbol
     */    
    public EnumType checkType(final String symbol, final EnumType[] type)
        throws SemanticException;    
    /** Checks if the specified data has one of the given types.
     * @param symbol the symbol to be checked
     * @param type an array containing the expected types
     * @throws SemanticException if the check fails, reporting the expected 
     * type if any
     * @return the type actually assigned to the symbol
     */
    public EnumType checkType(final Data symbol, final EnumType[] type)
        throws SemanticException;
    
    /** Returns the symbols contained in the current token list. */    
    public Data[] getSymbolList();
    
    /** Assigns the specified type to the given token.
     * @param token the given token
     * @param type the specified type
     * @throws SemanticException if the symbol is already declared
     * @return the specified type
     */    
    public EnumType put(final GenericToken token, final EnumType type)
        throws SemanticException;    
    /** Assigns the specified type to the given symbol. */    
    public EnumType put(final String symbol, final EnumType type)
        throws SemanticException;
    /** Assigns the specified type to the given data element. */    
    public EnumType put(final Data symbol, final EnumType type)
        throws SemanticException;

    /** Returns the type assigned to the given symbol.
     * @param symbol the specified symbol
     * @return the type assigned or <CODE>null</CODE> if none
     */    
    public EnumType get(final String symbol);
    /** Returns the type assigned to the given data. */    
    public EnumType get(final Data symbol);
    
    /** Changes the type assigned to the given data element with the specified one.
     * @param symbol the symbol to be re-assigned
     * @param type the new type
     * @return the previous type or <CODE>null</CODE> if none
     */    
    public EnumType changeType(final Data symbol, final EnumType type);
}
