/*
 * TaggedContent.java
 *
 * Created on 4 aprile 2004, 12.33
 */

package it.unive.dsi.pear.core.parser;

import java.util.*;

/** This class implements the checks on the definition of the tag set: every
 * message that has a tagged content can use this class to ensure the
 * respect of the correct tag use.
 * @author Francesco Placella
 */
public class TaggedContent
{
    private static final int DEFAULT_BUF_SIZE = 20;
    
    private String[][] tags;
    private Set[] availableTags;

    /** Creates a new instance of <CODE>TaggedContent</CODE>.
     * @param tags an array of string arrays each one containing a tag 
     * declaration
     */
    public TaggedContent(String[][] tags)
    {
        this.tags = tags;
        this.reset();
    }
    
    /** Resets the state of the tag set in order to match the initial values. */    
    public void reset() { this.availableTags = this.getDefinedTags(); }
    
    /** Checks if the given tag is allowed with respect to the current tag set
     * configuration.
     */    
    public boolean matchTag(String tag)
    {
        if (this.availableTags == null) return true;
        
        int size = this.availableTags.length;

        for (int i = 0; i < size; ++i)
        {
            Set tagSet = this.availableTags[i];
            if (tagSet != null && tagSet.remove(tag))
            {
                this.availableTags[i] = null;
                return true;
            }
        }

        return false;
    }

    /* getDefinedTags initializes the tag sets in order to match the initial
     * configuration.
     */
    private Set[] getDefinedTags()
    {
        if (this.tags == null) return null;
        
        int size = tags.length;
        HashSet[] res = new HashSet[size];
        
        for (int i = 0; i < size; ++i)
        {
            String[] t = tags[i];
            if (t != null) res[i] = new HashSet(Arrays.asList(t));
        }
        
        return res;
    }
    
    public String toString()
    {
       StringBuffer res = new StringBuffer(DEFAULT_BUF_SIZE);
       res.append("{ ");
       
       int size = this.availableTags.length;
       for (int i = 0; i < size; ++i)
       {
           Set at = this.availableTags[i];
           if (at != null) res.append(at.toString() + " ");
       }
       
       res.append("}");
       return res.toString();
    }
}
