/*
 * TaggedDataImpl.java
 *
 * Created on 7 marzo 2004, 13.14
 */

package it.unive.dsi.pear.core.parser;

import java.util.*;

/** This class is a straightforward implementation of the <CODE>TaggedData</CODE>
 * interface.
 * @author Francesco Placella
 */
public class TaggedDataImpl extends SimpleData implements TaggedData
{
    private String tag;
    
    /** Creates a new instance of <CODE>TaggedDataImpl</CODE>. */
    public TaggedDataImpl() {}
    /** Creates a new instance of <CODE>TaggedDataImpl</CODE>.
     * @param info the element to be tagged
     * @param tag the related tag
     */    
    public TaggedDataImpl(String info, String tag)
    {
        super(info);
        this.tag = tag;
    }
    
    public boolean putMatching(Map substMap, List iList)
    {
        Class rClass = this.getClass();
        ListIterator it = iList.listIterator();
        while (it.hasNext())
        {
            Object obj = it.next();
            if (obj != null && rClass == obj.getClass())
            {
                TaggedData iData = (TaggedData)obj;
                String tag = this.getTag();
                if (tag.equals(iData.getTag()) ||
                    tag.equals(TaggedData.ANY_TAG))
                {
                    List tdList = Arrays.asList(new Data[]{ iData });
                    if (super.putMatching(substMap, tdList))
                    {
                        it.set(null);
                        return true;
                    }
                }
            }
        }
        return false;
    }

    public String toString()
    {
        String dataStr = super.toString();
        if (tag != null) return dataStr + ":" + tag;
        else return dataStr;                
    }
    
    /** Getter for property tag.
     * @return Value of property tag.
     *
     */
    public java.lang.String getTag() { return tag; }
    
    /** Setter for property tag.
     * @param tag New value of property tag.
     *
     */
    public void setTag(java.lang.String tag) { this.tag = tag; }


    public boolean equals(Object obj)
    {
        if (!(obj instanceof TaggedData)) return false;
        
        String tag = ((TaggedData)obj).getTag();
        return (this.getTag().equals(tag) || TaggedData.ANY_TAG.equals(tag)) &&
                super.equals(obj);
    }
    
    public boolean containsTagged() { return true; }
    public int getTagNumber() { return 1; }
    
    public KeyType getKeyType() { return KeyType.NO_KEY; }
}
