/*
 * JNotifyingTextArea.java
 *
 * Created on 15 maggio 2004, 20.02
 */

package it.unive.dsi.pear;

/** This class implements a text area that notifies the change of its content.
 * @author Francesco Placella
 */
public class JNotifyingTextArea extends javax.swing.JTextArea
{
    /** The name of the property whose change is notified. */    
    public static final String PROPERTY_NAME = "text";
    
    /** Appends the given text to the end of the document notifying the event.
     * Overrides <CODE>JTextArea.append</CODE>.
     * @param text the text to be appended
     */    
    public void append(String text)
    {
        String oldText = null;
        super.append(text);
        this.firePropertyChange(PROPERTY_NAME, oldText, text);
    }
    
    /** Appends the given text to the end of the document without notifying
     * the event.
     * @param text the text to be appended.
     */    
    public final void silentAppend(String text) { super.append(text); }
    
    /** Sets the text of this text area to the specified text notifying the
     * event. Overrides <CODE>javax.swing.text.JTextComponent.setText</CODE>.
     * @param text the new text to be set
     */    
    public void setText(String text)
    {
        String oldText = null;
        super.setText(text);
        this.firePropertyChange(PROPERTY_NAME, oldText, text);
    }
}
