/*
 * JTaggingBox.java
 *
 * Created on 27 agosto 2004, 13.00
 */

package it.unive.dsi.pear;

import javax.swing.*;

import it.unive.dsi.pear.core.parser.Protocol;

/** This class implements a component allowing to choose the desired tagging of
 * the given protocol or its original source.
 * @author Francesco Placella
 */
public class JTaggingBox extends JComboBox
{
    private static final String TAGGING = "tagging";
    private static final String PROTOCOL = "protocol";
    
    private Protocol protocol;
    
    /** Creates a new instance of TaggingBox */
    public JTaggingBox()
    {
        this.setFocusable(false);
    }
    
    /** Sets the current protocol.
     * @param protocol the protocol to be set as current
     * @return the number of taggings available
     */    
    public int setProtocol(Protocol protocol)
    {
        this.protocol = protocol;
        return this.updateTaggings();
    }
    
    /** Updates the content of the component according to the current set of
     * taggings available for the protocol.
     * @return the number of available taggings
     */    
    public int updateTaggings()
    {
        int size = 0;
        
        java.awt.event.ActionListener[] al = this.getActionListeners();
        for (int i = 0; i < al.length; ++i) this.removeActionListener(al[i]);
        
        this.removeAllItems();
        
        if (protocol != null)
        {
            this.addItem(PROTOCOL);        
            Protocol[] taggings = protocol.getTaggings();
            if (taggings != null)
            {            
                size = taggings.length;
                for (int i = 0; i < size; ++i)
                    this.addItem(TAGGING + " " + (i + 1));
            }
        }

        for (int i = 0; i < al.length; ++i) this.addActionListener(al[i]);
        this.setEnabled(size > 0);
        
        return size;
    }
    
    /** Gets the selected tagging.
     * @return a tagging or the protocol according to which of the two is 
     * selected
     */    
    public Protocol getSelectedTagging()
    {
        int index = this.getSelectedIndex();
        return index == 0 ? protocol : protocol.getTaggings()[index - 1];
    }
    
    /** Enables or disables this component: there must be at leat one tagging
     * available. Overrides <CODE>java.awt.Component</CODE>.
     * @param enabled If true, this component is enabled; otherwise this
     * component is disabled
     */    
    public void setEnabled(boolean enabled)
    {
        if (!enabled || this.getItemCount() > 0) super.setEnabled(enabled);
    }
    
    /** Selects the current protocol. */    
    public final void selectProtocol()
    {
        if (this.getItemCount() > 0) this.setSelectedIndex(0);
    }
    
    /** Checks if the current protocol is selected.
     * @return <CODE>true</CODE> if the current protocol is selected,
     * <CODE>false</CODE> otherwise
     */    
    public final boolean isProtocolSelected()
    {
        return this.getSelectedIndex() == 0; 
    }
}
