/*
 * JTextAreaStream.java
 *
 * Created on 15 aprile 2004, 14.51
 */

package it.unive.dsi.pear;

import java.io.*;

import javax.swing.*;

/** This class implements a <CODE>PrintStream</CODE> that has associated a text
 * area: every byte written to thte stream is displayed onto the text area.
 * @author Francesco Placella
 */
public class JTextAreaStream extends PrintStream
{
    /** The associated textArea. */    
    protected JTextArea textArea = null;
    
    /** Creates a new instance of the class.
     * @param printStream an output stream on which are written the given bytes
     * @param textArea the associated text area
     */    
    public JTextAreaStream(OutputStream printStream, JTextArea textArea)
    {
        super(printStream);
        this.textArea = textArea;
    }
    
    /** Write len bytes from the specified byte array starting at offset off to
     * this stream. Overrides <CODE>PrintStream.write</CODE>.
     * @param buf a byte array
     * @param off offset from which to start taking bytes
     * @param len number of bytes to write
     */    
    public void write(byte buf[], int off, int len)
    {
        super.write(buf, off, len);
        if (textArea.isVisible()) textArea.append(new String(buf, off, len));
    }
    
    /** Clears the content of the text area. */    
    public final void clear()
    {
        textArea.setText(null);
    }
}