/*
 * checkType.java
 *
 * Created on 17 giugno 2004, 11.53
 */

package it.unive.dsi.pear.plugins;

import java.util.*;

import it.unive.dsi.pear.core.parser.AbstractData;
import it.unive.dsi.pear.core.parser.Container;
import it.unive.dsi.pear.core.parser.Data;
import it.unive.dsi.pear.core.parser.KeyInstance;
import it.unive.dsi.pear.core.parser.Instruction;
import it.unive.dsi.pear.core.DefaultOperation;

/**
 * This operation performs a type check of the given arguments.
 * @author Francesco Placella
 */
public class checkType extends DefaultOperation
{
    static final Data UNTRUSTED = AbstractData.ABSTRACT_DATA.newData("Un");
    
    private static final int MESSAGE_INDEX = 2;
    
    private static final String RUN = "run";
    private static final String NONCE = "nonce";
    private static final String KEY_PAIR = "key_asym";
    private static final String PUBLIC_KEY = "key_pub";
    private static final String PRIVATE_KEY = "key_priv";
    private static final String LONG_TERM_KEY = "key_sym";

    public Object perform(Container container, Container[] args)
        throws Container.OperationException
    {
        try
        {
            int size = args.length - 1;
            Data type = (Data)args[size];
            for (int i = 0; i < size; ++i)
                if (!this.perform(container, (Data)args[i], type))
                    return Boolean.FALSE;

            return Boolean.TRUE;
        }
        catch (Exception ex) { throw new Container.OperationException(ex); }
    }
    
    static boolean perform(Container container, Data data, Data type)
    
    {
        Iterator it = data.content().iterator();
        while (it.hasNext())
            if (!checkType(container, (Data)it.next(), type)) return false;
        
        return true;
    }
    
    private static boolean checkType(Container container, Data data, Data type)
    {
        boolean isKey = false;
        if (type instanceof Instruction)
        {
            String name = type.getName();
            isKey = name.equals(KEY_PAIR) || name.equals(LONG_TERM_KEY);
        }
        
        // PUBLIC KEY & PRIVATE KEY 
        if (isKey || data instanceof KeyInstance)
        {
            if (isKey) return keyPair(container, data, (Instruction)type);
            else if (data.isSimple()) // ?!
                if (keyDecl(container, data, type)) return true;
        }

        // PROJECTION
        return projection(container, data, type);
    }

    
    // -------------------------------------------- Environment Well-Formedness
    
    private static final 
        boolean projection(Container container, Data data, Data type)
    {
        if (!type.equals(container.get(data))) return false;
        return checkWellFormedness.perform(container, data, type);
    }
    

    // ---------------------------------------------- Typing Rules for Messages
    
    private static boolean keyPair(
        Container container, Data key, Instruction type)
    {
        if (key instanceof KeyInstance)
        {
            KeyInstance keyInst = (KeyInstance)key;
            String typeName = keyInst.isPublic() ? PUBLIC_KEY : PRIVATE_KEY;
            if (!type.getName().equals(typeName)) return false;
            type.setName(KEY_PAIR);
            key = keyInst.getKey();
        }
        
        if (checkType.projection(container, key, type)) return true;
        
        Data[] args = type.getData();
        if (args.length == 2)
        {
            // tries to swap type arguments        
            Data d = args[0];
            args[0] = args[1];
            args[1] = d;
            return checkType.projection(container, key, type);
        }
        
        return false;
    }

    private static boolean keyDecl(
        Container container, Data keyInst, Data type)
    {
        return type.equals(UNTRUSTED);
    }
}
