/*
 * CloneableMap.java
 *
 * Created on 26 marzo 2004, 15.34
 */

package it.unive.dsi.pear.util;

import java.util.*;

/**
 * This class implements a Map providing a non-shallow clone method
 * @author  Francesco Placella
 */
public class CloneableMap extends java.util.HashMap
    implements CloneableElement
{
    /** An empty map. */    
    public static final Map EMPTY_MAP = new CloneableMap(true, 0);
    
    private boolean shallow = false;
    
    public CloneableMap() {}
    public CloneableMap(boolean shallow) { this.shallow = shallow; }
    public CloneableMap(boolean shallow, int size)
    {
        super(size);
        this.shallow = shallow;
    }
    
    /** Returns a copy of the object: according to the parameter set at creation
     * time the clone method behaviour is shallow or recursive.
     */    
    public Object clone()
    {
        CloneableMap cm;
        
        if (shallow) cm = (CloneableMap)super.clone();
        else
        {
            cm = new CloneableMap(false, this.size());
            Iterator it = this.entrySet().iterator();
            while (it.hasNext())
            {
                Map.Entry entry = (Map.Entry)it.next();
                Object val = entry.getValue();

                if (val != null)
                {
                    val = ((CloneableElement)val).clone();
                    cm.put(entry.getKey(), val);
                }
            }
        }
        
        return cm;
    }
}
