/*
 * XPEAbstractNode.java
 *
 * Created on 5 gennaio 2005, 15.43
 */

package it.unive.dsi.pear.plugins.output.xpe;

import java.util.Iterator;

import it.unive.dsi.pear.core.parser.Data;
import it.unive.dsi.pear.core.parser.KeyData;
import it.unive.dsi.pear.util.ArrayIterator;

import org.w3c.dom.Document;
import org.w3c.dom.Element;


/**
 *
 * @author Francesco Placella
 */
abstract public class XPEAbstractNode implements XPENode
{
    static final String NS_URI = "http://www.dsi.unive.it/~maffei/pear";
    
    // key type
    static final String KEY_TYPE = "key-type";
    static final String KT_TYPE = "type";
    static final String KT_SYM = "symmetric";
    static final String KT_ASYM = "asymmetric";


// ----------------------------------------------------------------------------

    private Document xpe;
    
    private static final int INDENT = 2;
    
    abstract public String getElementName();
    abstract protected void buildElement(Element e);

    public Element getElement()
    {
        Element e = this.createElement(this.getElementName());
        this.buildElement(e);
        e.normalize();
        return e;
    }
    
    protected final Element createElement(String name) { return xpe.createElementNS(NS_URI, name); }
    
    protected Element createElement(String name, Element parent, int indentLevel)
    {
        this.indent(parent, indentLevel);
        Element ne = this.createElement(name);
        parent.appendChild(ne);
        return ne;
    }
    
    protected void indent(Element e, int level)
    {
        int size = level * INDENT;
        StringBuffer buf = new StringBuffer(size + 1);
        buf.append("\n");
        for (int i = 0; i < size; ++i) buf.append(" ");
        e.appendChild(xpe.createTextNode(buf.toString()));
    }
    
    protected void appendXPE(XPENode xn, Element parent, int indentLevel)
    {
        xn.setDocument(xpe);
        this.indent(parent, indentLevel);
        parent.appendChild(xn.getElement());
    }
    
    protected final void appendText(Element parent, String text)
    {
        parent.appendChild(xpe.createTextNode(text));
    }
    
    protected void appendKeyType(Element e, KeyData key, int indentLevel)
    {
        Element kt = this.createElement(KEY_TYPE, e, indentLevel);
        Data[] pr = key.getPrincipals();
        kt.setAttribute(KT_TYPE, pr.length > 1 ? KT_SYM : KT_ASYM);
        this.appendData(kt, pr, indentLevel + 1);
        this.indent(kt, indentLevel);
    }
    
    protected final void appendData(Element e, Data[] ids, int indentLevel)
    {
        this.appendData(e, new ArrayIterator(ids), indentLevel);
    }
    
    protected void appendData(Element e, Iterator it, int indentLevel)
    {
        while (it.hasNext())
        {
            XPENode xn = new XPEData((Data)it.next(), indentLevel);
            this.appendXPE(xn, e, indentLevel);
        }
    }

    public void setDocument(Document document) { xpe = document; }
}
