/*
 * XPECiphertext.java
 *
 * Created on 6 gennaio 2005, 19.45
 */

package it.unive.dsi.pear.plugins.output.xpe;

import it.unive.dsi.pear.core.parser.Ciphertext;
import it.unive.dsi.pear.core.parser.Data;
import it.unive.dsi.pear.core.parser.KeyInstance;

import org.w3c.dom.Element;

/**
 *
 * @author Francesco Placella
 */
public class XPECiphertext extends XPEAbstractNode
{
    private static final String ELEMENT = "message";
    private static final String EL_NAME = "name";
    private static final String EL_KEY = "key";
    private static final String EL_ENCRYPTED = "encrypted";
    private static final String EL_ENC = "encryption";
    private static final String EL_ENC_NONE = "none";
    private static final String EL_ENC_SYM = "symmetric";
    private static final String EL_ENC_PRIV = "private";
    private static final String EL_ENC_PUB = "public";

// ----------------------------------------------------------------------------
    
    private Ciphertext message;
    private int indentLevel;
    
    /** Creates a new instance of XPECiphertext */
    public XPECiphertext(Ciphertext c, int indentLevel)
    {
        message = c;
        this.indentLevel = indentLevel;
    }
    
    public final String getElementName() { return ELEMENT; }

    protected void buildElement(Element e)
    {
        String name = message.getName();
        if (name != null) e.setAttribute(EL_NAME, name);
        
        Data key = message.getKey();
        
        if (key != null)
        {
            e.setAttribute(EL_KEY, key.getName());
            e.setAttribute(EL_ENCRYPTED, String.valueOf(message.isEncrypted()));

            if (message.isAsymmetric())
            {
                KeyInstance ki = (KeyInstance)message.getKey();
                if (ki.isPublic()) e.setAttribute(EL_ENC, EL_ENC_PUB);
                else e.setAttribute(EL_ENC, EL_ENC_PRIV);
            }
            else e.setAttribute(EL_ENC, EL_ENC_SYM);
        }
        else
           e.setAttribute(EL_ENC, EL_ENC_NONE);
        
        this.appendData(e, message.getContent(), indentLevel + 1);
        
        this.indent(e, indentLevel);
    }
}
