/*
 * XPEData.java
 *
 * Created on 6 gennaio 2005, 0.58
 */

package it.unive.dsi.pear.plugins.output.xpe;

import it.unive.dsi.pear.core.parser.Ciphertext;
import it.unive.dsi.pear.core.parser.Data;
import it.unive.dsi.pear.core.parser.Instruction;
import it.unive.dsi.pear.core.parser.MultiTaggedData;
import it.unive.dsi.pear.core.parser.ParametricTaggedData;
import it.unive.dsi.pear.core.parser.TaggedData;
import it.unive.dsi.pear.core.parser.ValueData;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 *
 * @author Franceso Placella
 */
public class XPEData extends XPEAbstractNode
{
    private static final String EL_IDENTIFIER = "identifier";
    private static final String EL_NAME = "name";
    private static final String EL_TAG = "tag";
    
    private static final String TP_TAG_PARAMS = "tag-parameters";    
    
// ----------------------------------------------------------------------------
    
    private XPENode node;
    private Data data;
    private int indentLevel;
    
    /** Creates a new instance of XPEData */
    public XPEData(Data d, int indentLevel)
    {
        this.indentLevel = indentLevel;
        this.initData(d);
    }
    
    public final Element getElement()
    {
        return node != this ? node.getElement() : super.getElement();
    }
    
    protected void buildElement(Element e)
    {
        e.setAttribute(EL_NAME, data.getInfo());
        
        if (data instanceof TaggedData)
        {
            TaggedData td = (TaggedData)data;
            e.setAttribute(EL_TAG, td.getTag());
            
            if (td instanceof ParametricTaggedData)
            {
                ParametricTaggedData ptd = (ParametricTaggedData)td;
                int cil = indentLevel + 1;
                Element tp = this.createElement(TP_TAG_PARAMS, e, cil);
                this.appendData(tp, ptd.getParameters(), cil + 1);
                this.indent(tp, cil);
                this.indent(e, indentLevel);
            }
        }
    }
    
    public final String getElementName() { return EL_IDENTIFIER; }
    
    public final void setDocument(Document xpe)
    {
        super.setDocument(xpe);
        if (node != this) node.setDocument(xpe);
    }
    
// ----------------------------------------------------------------------------

    private final void initData(Data d)
    {
        data = d;
        if (d instanceof Ciphertext)
        {
            Ciphertext c = (Ciphertext)d;
            Data[] content = c.getContent();
            if (content.length == 1 && content[0] instanceof Instruction)
                node = new XPEInstruction(c, indentLevel);
            else
                node = new XPECiphertext(c, indentLevel);
        }
        else if (d instanceof ValueData)
            node = new XPEValueData((ValueData)d, indentLevel);
        else if (d instanceof Instruction)
            node = new XPEInstruction((Instruction)d, indentLevel);
        else if (d instanceof MultiTaggedData)
            node = new XPEMultiTaggedData((MultiTaggedData)d, indentLevel);
        else node = this;
    }
}
