/*
 * XPEInstructionNode.java
 *
 * Created on 6 gennaio 2005, 18.27
 */

package it.unive.dsi.pear.plugins.output.xpe;

import it.unive.dsi.pear.core.InstructionDecoration;
import it.unive.dsi.pear.core.parser.Conditions;
import it.unive.dsi.pear.core.parser.Container;
import it.unive.dsi.pear.core.parser.Instruction;
import it.unive.dsi.pear.core.parser.InstructionNode;
import it.unive.dsi.pear.core.parser.Operations;
import it.unive.dsi.pear.core.parser.Rule;
import java.util.Collection;

import java.util.Iterator;

import org.w3c.dom.DOMException;
import org.w3c.dom.Element;

/**
 *
 * @author Francesco Placella
 */
public class XPEInstructionNode extends XPERuleNode
{
    // sequential process
    static final String NODE = "instruction-node";
    static final String NULL_NODE = "null-node";
    static final String EL_LABEL = "label";
    static final int EL_INDENT = XPESequentialProcess.DF_INDENT + 1;

    // signature
    static final String INSTRUCTION = "instruction";
    static final String IN_NAME = "name";
    static final int IN_INDENT = EL_INDENT + 1;
    
    // nil
    static final String NIL = "nil";
    
    // rule
    static final String RULE = "rule";
    static final String RL_NAME = "name";
    static final int RL_INDENT = EL_INDENT + 1;
    static final int CL_INDENT = RL_INDENT + 1;
    
    // containers
    static final String CONTAINERS = "containers";
    static final int CS_INDENT = EL_INDENT + 1;
    static final int CN_INDENT = CS_INDENT + 1;

    
// ----------------------------------------------------------------------------
    
    private InstructionNode inode;
    private boolean nil;
    private boolean hasChildren = false;
    
    /** Creates a new instance of XPESequentialProcess */
    public XPEInstructionNode(InstructionNode in)
    {
        super(in, EL_INDENT);
        
        inode = in;
        nil = inode.getInstruction().isNil();
    }
    
    public final String getElementName()
    {
        return this.isNeedingNode() ? (nil ? NULL_NODE : NODE) : (nil ? NIL : INSTRUCTION);
    }
    
    protected void buildElement(Element e)
    {
        this.appendInstruction(e);

        if (this.hasDecoration())
        {
            this.appendRule(e);
            this.appendContainers(e);
        }
        
        if (this.isNeedingNode()) this.indent(e, EL_INDENT);
    }

    protected XPENode getXPEChild(Object child)
    {
        if (child instanceof Conditions)
            return new XPEConditions((Conditions)child);
        else if (child instanceof Operations)
            return new XPEActions((Operations)child);
        else
            return null;
    }
    
// ----------------------------------------------------------------------------
    
    private final boolean hasDecoration()
    {
        Object deco = inode.getDecoration();
        return deco != null && deco instanceof InstructionDecoration;
    }
    
    private final boolean isNeedingNode()
    {
        return inode.getInstrLabel() != null || this.hasDecoration();
    }

    private void appendInstruction(Element e)
    {
        int indentLevel = EL_INDENT;
        Element parent = e;
        
        if (this.isNeedingNode()) 
            parent = this.createElement(nil ? NIL : INSTRUCTION, e, indentLevel = IN_INDENT);
        
        String label = inode.getInstrLabel();
        if (label != null) e.setAttribute(EL_LABEL, label);
        
        if (!nil)
        {
            Instruction instr = inode.getInstruction();
            parent.setAttribute(IN_NAME, instr.getName());
            this.appendData(parent, instr.getData(), indentLevel + 1);
            this.indent(parent, indentLevel);
        }
    }

    private void appendRule(Element e)
    {
        InstructionDecoration id = (InstructionDecoration)inode.getDecoration();
        Rule rule = id.getRule();
        if (rule == null) return;

        Element rl = this.createElement(RULE, e, RL_INDENT);
        rl.setAttribute(RL_NAME, id.getRuleName());

        this.appendXPEChildren(rl, rule, CL_INDENT);
    }
    
    private void appendContainers(Element e)
    {
        InstructionDecoration id = (InstructionDecoration)inode.getDecoration();
        Collection c = id.getContainers();
        if (c == null) return;

        Element cs = this.createElement(CONTAINERS, e, CS_INDENT);
        
        Iterator it = c.iterator();
        while (it.hasNext())
        {
            Container cn = (Container)it.next();
            this.appendXPE(new XPEContainer(cn), cs, CN_INDENT);
        }
        
        this.indent(cs, CS_INDENT);
    }
}