/*
 * XPESequentialProcess.java
 *
 * Created on 6 gennaio 2005, 15.16
 */

package it.unive.dsi.pear.plugins.output.xpe;

import java.util.*;

import it.unive.dsi.pear.core.parser.*;

import org.w3c.dom.*;

/**
 *
 * @author Franceso Placella
 */
public class XPESequentialProcess extends XPEAbstractNode
{
    // sequential process
    static final String ELEMENT = "sequential-process";
    static final String EL_NAME = "name";
    static final int EL_INDENT = XPEProtocol.SP_INDENT;

    // signature
    static final String SIGNATURE = "signature";
    static final String SG_PRINCIPAL = "principal";
    static final int SG_INDENT = EL_INDENT + 1;

    // declaration
    static final String DECLARATION = "declaration";
    static final String DC_IDENTIFIER = "identifier";
    static final String DC_TYPE = "type";
    static final String DC_TYPE_ID = "Id";
    static final int DC_INDENT = SG_INDENT + 1;
    static final int KT_INDENT = DC_INDENT + 1;

    // definition
    static final String DEFINITION = "definition";
    static final int DF_INDENT = EL_INDENT + 1;
    static final int IN_INDENT = DF_INDENT + 1;

    
// ----------------------------------------------------------------------------
    
    private SequentialProcess seqproc;
    
    /** Creates a new instance of XPESequentialProcess */
    public XPESequentialProcess(SequentialProcess sp) { seqproc = sp; }
    
    public final String getElementName() { return ELEMENT; }
    
    protected void buildElement(Element e)
    {
        e.setAttribute(EL_NAME, seqproc.getDeclaration().getName());

        this.appendSignature(e);
        this.appendDefinition(e);
        this.indent(e, EL_INDENT);
    }


// ----------------------------------------------------------------------------

    private void appendSignature(Element e)
    {
        Element sg = this.createElement(SIGNATURE, e, SG_INDENT);
        Iterator it = seqproc.declIterator();
        Data d = (Data)it.next();
        sg.setAttribute(SG_PRINCIPAL, d.getInfo());
        
        while (it.hasNext())
        {
            d = (Data)it.next();
            Element dc = this.createElement(DECLARATION, sg, DC_INDENT);
            dc.setAttribute(DC_IDENTIFIER, d.getInfo());
            
            if (d instanceof KeyData)
            {
                this.appendKeyType(dc, (KeyData)d, KT_INDENT);
                this.indent(dc, DC_INDENT);
            }
            else dc.setAttribute(DC_TYPE, DC_TYPE_ID);
        }
        
        this.indent(sg, SG_INDENT);
    }
    
    private void appendDefinition(Element e)
    {
        Element df = this.createElement(DEFINITION, e, DF_INDENT);
        
        Iterator it = seqproc.instrIterator();
        while (it.hasNext())
        {
            InstructionNode in = (InstructionNode)it.next();
            this.appendXPE(new XPEInstructionNode(in), df, XPEInstructionNode.EL_INDENT);
        }
        
        this.indent(df, DF_INDENT);
    }

}
