/*
 * XPEOutputTab.java
 *
 * Created on 10 gennaio 2005, 1.03
 */

package it.unive.dsi.pear.plugins.tabber.xpe;

import it.unive.dsi.pear.Pear;
import it.unive.dsi.pear.core.Configuration;
import it.unive.dsi.pear.core.parser.Protocol;
import it.unive.dsi.pear.log.Log;
import it.unive.dsi.pear.plugins.output.OutputPlugin;
import it.unive.dsi.pear.plugins.output.xpe.XPEOutput;
import it.unive.dsi.pear.util.DefaultTreeNode;
import it.unive.dsi.pear.util.ExtensionBasedFileFilter;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Insets;
import java.io.BufferedInputStream;
import java.io.BufferedWriter;
import java.io.CharArrayWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.FilterReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;
import java.io.Writer;
import java.net.URL;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JTree;
import javax.swing.filechooser.FileFilter;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;
import org.w3c.dom.ProcessingInstruction;


/**
 *
 * @author Francesco Placella
 */
public class XPEOutputTab extends javax.swing.JPanel
    implements it.unive.dsi.pear.plugins.tabber.TabPlugin
{
    private static final String PATH = "resources/";
    private static final String BUILTIN = PATH + "builtin-stylesheets.ini";
    private static final String XSL_EXT = ".xsl";
    private static final String XPE_EXT = ".xpe";
    private static final String HTM_EXT = ".html";
    private static final String NAME = "xpe";
    private static final String TOOL_TIP = "an XML-based protocol encoder";
    private static final String ENCODING = "utf-8";
    
    private static final Font CONSOLE_FONT = Pear.getConsoleFont();
    private static final Font DEFAULT_FONT = Pear.getDefaultFont();
    private static final Insets SOURCE_MARGIN = new Insets(2, 5, 2, 5);
    private static final Dimension DIALOG_BUTTON_SIZE = new Dimension(65, 25);
    private static final Insets BUTTON_MARGIN = new Insets(2, 2, 2, 2);
    private static final DefaultTreeCellRenderer RENDERER = new DefaultTreeCellRenderer();
    
    private static final JFileChooser fileChooser;
    private static final FileFilter xslFilter;
    
    private static TransformerFactory transformerFactory;
    private static DocumentBuilder dBuilder;
    
    static
    {
        fileChooser = new JFileChooser();
        fileChooser.setCurrentDirectory(new File(".."));
        xslFilter = new ExtensionBasedFileFilter("xsl");
        
        URL iu = Configuration.getCurrent().getResource(PATH + "xsleaf.gif");
        ImageIcon ii = new ImageIcon(iu);
        RENDERER.setFont(DEFAULT_FONT);
        RENDERER.setLeafIcon(ii);
        
        try { transformerFactory = TransformerFactory.newInstance(); }
        catch (Exception e) { e.printStackTrace(); }
        
        try
        {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            dbf.setNamespaceAware(true);
            dBuilder = dbf.newDocumentBuilder();
        }
        catch (Exception e) { e.printStackTrace(); }
    }


    /**
     * Holds value of property currentSatus.
     */
    private Protocol protocol;
    
    private final OutputPlugin xpe = new XPEOutput();
    private final CharArrayWriter result = new CharArrayWriter();
    private final Configuration conf = Configuration.getCurrent();

    private Document xpeOutput;
    private Document xslStylesheet;
    private ProcessingInstruction xsl;
    private TreePath lastSelection;
    private File xslFile;
    private Log log;
    private String xSource;


    /** Creates new form XPEOutputTab */
    public XPEOutputTab() { this.initComponents(); }
    public void setLog(Log log) { this.log = log; }    
    
    private class SkipCharReader extends FilterReader
    {
        private final String skipped;
                
        private SkipCharReader(Reader r, String skipped)
        {
            super(r);
            this.skipped = skipped;
        }
        
        public final int read() throws IOException
        {
            int ch = super.read();
            while (skipped.indexOf((char)ch) != -1) ch = super.read();
            return ch;
        }
    }
    
    
    private final class BuiltinStylesheetRoot extends DefaultTreeNode
    {
        private BuiltinStylesheetRoot(Collection stylesheets)
        {
            int i = 0;
            Iterator it = stylesheets.iterator();
            while (it.hasNext())
            {
                String ss = (String)it.next();
                this.addChild(new BuiltinStylesheetNode(ss), i++);
            }
        }
    }
    
    private final class BuiltinStylesheetNode extends DefaultTreeNode
    {
        private String name;
        private BuiltinStylesheetNode(String name) { this.name = name; }
        public String toString() { return name; }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        scrollPane = new javax.swing.JScrollPane();
        source = new javax.swing.JEditorPane();
        southPanel = new javax.swing.JPanel();
        swPanel = new javax.swing.JPanel();
        outputLabel = new javax.swing.JLabel();
        output = new javax.swing.JTextField();
        browseOut = new javax.swing.JButton();
        ssLabel = new javax.swing.JLabel();
        stylesheet = new javax.swing.JTextField();
        browse = new javax.swing.JButton();
        builtinCB = new javax.swing.JCheckBox();
        sePanel = new javax.swing.JPanel();
        export = new javax.swing.JButton();
        ssPanel = new javax.swing.JPanel();
        transform = new javax.swing.JToggleButton();
        protocolName = new javax.swing.JLabel();
        bsView = new javax.swing.JScrollPane();
        builtinStylesheets = new JTree(new BuiltinStylesheetRoot(this.getBuiltinStylesheets()));
        builtinStylesheets.setCellRenderer(RENDERER);
        builtinStylesheets.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

        setLayout(new java.awt.BorderLayout());

        setFont(DEFAULT_FONT);
        source.setBackground(javax.swing.UIManager.getDefaults().getColor("TextField.inactiveBackground"));
        source.setEditable(false);
        source.setFont(CONSOLE_FONT);
        source.setMargin(SOURCE_MARGIN);
        scrollPane.setViewportView(source);

        add(scrollPane, java.awt.BorderLayout.CENTER);

        southPanel.setLayout(new java.awt.BorderLayout());

        swPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        swPanel.setFont(DEFAULT_FONT);
        outputLabel.setFont(DEFAULT_FONT);
        outputLabel.setText("output file:");
        swPanel.add(outputLabel);

        output.setFont(DEFAULT_FONT);
        output.setPreferredSize(new java.awt.Dimension(200, 25));
        swPanel.add(output);

        browseOut.setFont(DEFAULT_FONT);
        browseOut.setText("browse");
        browseOut.setMargin(BUTTON_MARGIN);
        browseOut.setPreferredSize(DIALOG_BUTTON_SIZE);
        browseOut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                browseOutActionPerformed(evt);
            }
        });

        swPanel.add(browseOut);

        ssLabel.setFont(DEFAULT_FONT);
        ssLabel.setText("     stylesheet:");
        swPanel.add(ssLabel);

        stylesheet.setEditable(false);
        stylesheet.setFont(DEFAULT_FONT);
        stylesheet.setPreferredSize(new java.awt.Dimension(200, 25));
        swPanel.add(stylesheet);

        browse.setFont(DEFAULT_FONT);
        browse.setText("browse");
        browse.setMargin(BUTTON_MARGIN);
        browse.setPreferredSize(DIALOG_BUTTON_SIZE);
        browse.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                browseActionPerformed(evt);
            }
        });

        swPanel.add(browse);

        builtinCB.setFont(DEFAULT_FONT);
        builtinCB.setText("built-in");
        builtinCB.setActionCommand("builtin");
        builtinCB.setEnabled(false);
        swPanel.add(builtinCB);

        southPanel.add(swPanel, java.awt.BorderLayout.WEST);

        sePanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT, 0, 3));

        export.setFont(DEFAULT_FONT);
        export.setText("export");
        export.setMargin(BUTTON_MARGIN);
        export.setPreferredSize(DIALOG_BUTTON_SIZE);
        export.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exportActionPerformed(evt);
            }
        });

        sePanel.add(export);

        southPanel.add(sePanel, java.awt.BorderLayout.EAST);

        ssPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT, 5, 3));

        transform.setFont(DEFAULT_FONT);
        transform.setText("transform");
        transform.setMargin(BUTTON_MARGIN);
        transform.setPreferredSize(DIALOG_BUTTON_SIZE);
        transform.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                transformActionPerformed(evt);
            }
        });

        ssPanel.add(transform);

        southPanel.add(ssPanel, java.awt.BorderLayout.CENTER);

        add(southPanel, java.awt.BorderLayout.SOUTH);

        protocolName.setFont(CONSOLE_FONT);
        protocolName.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(1, 1, 1, 1)));
        protocolName.setPreferredSize(new java.awt.Dimension(34, 25));
        add(protocolName, java.awt.BorderLayout.NORTH);

        builtinStylesheets.setPreferredSize(new java.awt.Dimension(132, 48));
        builtinStylesheets.setRootVisible(false);
        builtinStylesheets.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                builtinStylesheetsMouseClicked(evt);
            }
        });
        builtinStylesheets.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                builtinStylesheetsValueChanged(evt);
            }
        });

        bsView.setViewportView(builtinStylesheets);

        add(bsView, java.awt.BorderLayout.EAST);

    }//GEN-END:initComponents

    private void browseOutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_browseOutActionPerformed

        fileChooser.setDialogTitle("choose output file");
        fileChooser.setFileFilter(fileChooser.getAcceptAllFileFilter());
        fileChooser.setSelectedFile(new File(output.getText()));
        if (fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
            output.setText(fileChooser.getSelectedFile().getAbsolutePath());
        
    }//GEN-LAST:event_browseOutActionPerformed

    private void browseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_browseActionPerformed
        
        fileChooser.setDialogTitle("choose stylesheet");
        fileChooser.setFileFilter(xslFilter);
        if (fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
        {
            xslFile = fileChooser.getSelectedFile();
            this.setStylesheet(xslFile.getName());
            builtinCB.setSelected(false);
            builtinStylesheets.clearSelection();
            lastSelection = null;
        }
        
    }//GEN-LAST:event_browseActionPerformed

    private void builtinStylesheetsMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_builtinStylesheetsMouseClicked

        if (!transform.isSelected())
        {
            int selRow = builtinStylesheets.getRowForLocation(evt.getX(), evt.getY());
            if (selRow == -1) builtinStylesheets.clearSelection();
        }
    }//GEN-LAST:event_builtinStylesheetsMouseClicked

    private void builtinStylesheetsValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_builtinStylesheetsValueChanged
        
        Object node = builtinStylesheets.getLastSelectedPathComponent();

        if (node != null)
        {
            lastSelection = builtinStylesheets.getSelectionPath();
            builtinCB.setSelected(true);
            this.setStylesheet(node.toString());
        }
        else if (builtinCB.isSelected())
        {
            try { if (xpeOutput != null && xsl != null) xpeOutput.removeChild(xsl); }
            catch (Exception e) { log.error(e); }
            xsl = null;

            this.setStylesheet(null);
            builtinCB.setSelected(false);
        }
        
    }//GEN-LAST:event_builtinStylesheetsValueChanged

    private void exportActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exportActionPerformed
        
        String fileName = output.getText();
        if (fileName.length() > 0)
        {
            Writer w = null;
            Reader r = new StringReader(xSource);
            try 
            {
                w = new BufferedWriter(new FileWriter(fileName));
                while (true)
                {
                    int ch = r.read();
                    if (ch == -1) break;
                    w.write(ch);
                }
            }
            catch (IOException ioe) { log.error(ioe); }
            finally
            {
                try { r.close(); } catch (IOException ioe) { log.error(ioe); }
                try { w.close(); } catch (IOException ioe) { log.error(ioe); }
            }
        }
        
    }//GEN-LAST:event_exportActionPerformed

    private void transformActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_transformActionPerformed
        
        try
        {
            boolean b = !transform.isSelected();
            this.transform(b ? null : this.getStylesheetSource());
            
            if (b) 
            {
                if (lastSelection != null) builtinStylesheets.setSelectionPath(lastSelection);
                else builtinStylesheets.clearSelection();
            }
        }
        catch (Exception e) { log.error(e); }
        
    }//GEN-LAST:event_transformActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton browse;
    private javax.swing.JButton browseOut;
    private javax.swing.JScrollPane bsView;
    private javax.swing.JCheckBox builtinCB;
    private javax.swing.JTree builtinStylesheets;
    private javax.swing.JButton export;
    private javax.swing.JTextField output;
    private javax.swing.JLabel outputLabel;
    private javax.swing.JLabel protocolName;
    private javax.swing.JScrollPane scrollPane;
    private javax.swing.JPanel sePanel;
    private javax.swing.JEditorPane source;
    private javax.swing.JPanel southPanel;
    private javax.swing.JLabel ssLabel;
    private javax.swing.JPanel ssPanel;
    private javax.swing.JTextField stylesheet;
    private javax.swing.JPanel swPanel;
    private javax.swing.JToggleButton transform;
    // End of variables declaration//GEN-END:variables
    

// ----------------------------------------------------------------------------

    public final String getToolTip() { return TOOL_TIP; }
    public final String getTabTitle() { return NAME; }
    public final Component getTab() { return this; }

    public void protocolDecorated(Protocol currentProtocol)
    {
        if (currentProtocol == protocol) protocol = null;
    }
    
    public void tabSelected(Protocol currentStatus)
    {
        if (currentStatus == null) this.clear();
        else
        {
            if (currentStatus != protocol)
            {
                protocol = currentStatus;
                
                protocolName.setText(protocol.getProtocolName() + " (XPE encoding)");
                this.updateOutputFile();
                
                xpeOutput = (Document)xpe.getOutput(protocol);
                this.updateXSLData();

                transform.setSelected(false);
                this.enableTransform(true);
                export.setEnabled(true);

                this.updateSource();
            }
        }
    }
    
    
    private final void updateSource() { this.transform(null); }
    private void transform(Source stylesource)
    {
        if (xpeOutput == null) return;
        
        try
        {
            result.reset();
            
            Transformer t =
                stylesource != null ?
                transformerFactory.newTransformer(stylesource) : 
                transformerFactory.newTransformer();
            
            this.updateXSLData();
            t.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM, XPEOutput.DTD_NAME);
            t.setOutputProperty(OutputKeys.ENCODING, ENCODING);
            t.transform(new DOMSource(xpeOutput), new StreamResult(result));
            
            Reader r = new StringReader(xSource = result.toString());
            source.setContentType(t.getOutputProperty(OutputKeys.MEDIA_TYPE));
            source.read(new SkipCharReader(r, "\r"), null);
            source.setCaretPosition(0);
        }
        catch (Exception te)
        {
            result.flush();
            log.error("XML serialization failed", te);
            System.err.println("\ndisplaying buffer content:\n\n" + result.toString());
        }
    }
    
    private Source getStylesheetSource()
    {
        if (xslStylesheet == null)
        {
            try
            {
                InputStream is = builtinCB.isSelected() ? 
                    conf.getResourceAsStream(PATH + stylesheet.getText()) :
                    new FileInputStream(xslFile);

                is = new BufferedInputStream(is);
                if (dBuilder == null) return new StreamSource(is);
                xslStylesheet = dBuilder.parse(is);
                is.close();
            }
            catch (Exception e)
            {
                log.error("XSL stylesheet has errors or doesn't exist", e);
                return null;
            }
        }

        return new DOMSource(xslStylesheet);
    }
    
    private final String getXSLData(String href)
    {
        return "type=\"text/xsl\" href=\"" + href + "\"";
    }
    
    private void updateXSLData()
    {
        String ss = stylesheet.getText();
        if (ss.length() > 0)
        {
            ss = this.getXSLData(ss);
            if (xsl == null)
            {
                xsl = xpeOutput.createProcessingInstruction("xml-stylesheet", ss);
                xpeOutput.insertBefore(xsl, xpeOutput.getDocumentElement());
            }
            else xsl.setData(ss);
        }
    }
    
    private void clear()
    {
        xsl = null;
        xpeOutput = null;
        protocol = null;
        source.setText(null);
        protocolName.setText(null);
        output.setText(null);
        export.setEnabled(false);
        transform.setEnabled(false);
    }
    
    private Collection getBuiltinStylesheets()
    {
        try
        {
            List l = conf.getBuiltinResources(BUILTIN, XSL_EXT, true);
            
            ListIterator it = l.listIterator();
            while (it.hasNext())
            {
                String ss = (String)it.next();
                it.set(ss.substring(ss.lastIndexOf('/') + 1));
            }
            
            return l;
        }
        catch (IOException ioe)
        {
            log.error(ioe);
            throw new NullPointerException();
        }
    }
    
    private final void updateOutputFile()
    {
        output.setText(protocol.getProtocolName() + (transform.isSelected() ? HTM_EXT : XPE_EXT));
        output.setCaretPosition(0);
    }
    
    private final void setStylesheet(String txt)
    {
        xslStylesheet = null;
        stylesheet.setText(txt);
        this.enableTransform(true);
        this.transform(txt != null && transform.isSelected() ? this.getStylesheetSource() : null);
    }
    
    private final void enableTransform(boolean b)
    {
        String s;
        transform.setEnabled(b && xpeOutput != null && (s = stylesheet.getText()) != null && s.length() > 0);
    }
}
