<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet version="1.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xpe="http://www.dsi.unive.it/~maffei/pear">
  
  <xsl:output method="text" version="1.0" encoding="UTF-8" indent="no" omit-xml-declaration="yes" media-type="text/plain"/>
  <xsl:strip-space elements="*"/>

  <xsl:variable name="gj-suffix">_GJ</xsl:variable>
  <xsl:variable name="P" select="concat(/xpe:protocol-encoding/xpe:protocol/@name, $gj-suffix)" />

  <xsl:template match="xpe:description" />

  <xsl:template match="xpe:protocol">
    <xsl:value-of select="$P" />
<xsl:text> :=
</xsl:text>
    <xsl:apply-templates select="xpe:instantiation" />
    <xsl:apply-templates select="xpe:sequential-process" />
  </xsl:template>
  
<!-- instantiation -->
  
  <xsl:template match="xpe:instantiation" >
    <xsl:apply-templates select="xpe:key-binding" />
    <xsl:text>(</xsl:text> <xsl:apply-templates select="xpe:sequential-process-calls" /> <xsl:text>)</xsl:text>
  </xsl:template>
  
  <!-- SYM-KEY / ASYM-KEY -->
  <xsl:template match="xpe:key-binding">
    <xsl:variable name="key" select="@key" />
    <xsl:variable name="key-decl" select="..//xpe:call[1]/xpe:identifier[@name = $key]/.."  />
    <xsl:variable name="sp-name" select="$key-decl/@sequential-process"  />
    <xsl:variable name="id-pos">
      <xsl:for-each select="$key-decl/xpe:identifier"> <xsl:if test="@name = $key"> <xsl:value-of select="position()" /> </xsl:if> </xsl:for-each>
    </xsl:variable>
    
    <xsl:variable name="formal-key-decl" select="/.//xpe:sequential-process[@name = $sp-name]//xpe:declaration[number($id-pos)]" />
    <xsl:variable name="formal-key" select="$formal-key-decl/@identifier" />

    <xsl:text>new(</xsl:text> <xsl:value-of select="$formal-key" /> <xsl:text> : </xsl:text>
    <xsl:call-template name="F_key">
      <xsl:with-param name="key" select="$formal-key" />
      <xsl:with-param name="key-type" select="$formal-key-decl/xpe:key-type" />
    </xsl:call-template>
    <xsl:text>).</xsl:text>

<xsl:text>
</xsl:text>
  </xsl:template>
  
  <!-- REPLICATION / PAR -->
  <xsl:template match="xpe:call">
    <xsl:if test="boolean(@replicated)"> <xsl:text>!</xsl:text> </xsl:if>
    <xsl:value-of select="@sequential-process" />
    <xsl:value-of select="$gj-suffix" />
    <xsl:call-template name="vert" />
  </xsl:template>

<!-- definition -->

  <!-- IDENTITY -->
  <xsl:template match="xpe:sequential-process">
<xsl:text>

</xsl:text>
    <xsl:value-of select="@name" />
    <xsl:value-of select="$gj-suffix" />
    <xsl:text> :=
  </xsl:text>
    <xsl:apply-templates select="xpe:definition" />
  </xsl:template>

  <xsl:template match="xpe:instruction-node">
    <xsl:variable name="out"> <xsl:apply-templates select="xpe:instruction" /> </xsl:variable>
    <xsl:if test="$out != ''">
      <xsl:value-of select="$out" />
      <xsl:if test="position() != last() - 1">
<xsl:text>.
  </xsl:text>
      </xsl:if>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="xpe:rule" />
  <xsl:template match="xpe:containers" />
  
<!-- instruction -->  
  
  <xsl:template match="xpe:instruction">
    <xsl:variable name="name" select="@name" />
    <xsl:choose>
      
      <!-- INPUT -->
      <xsl:when test="$name = 'in'">
        <xsl:value-of select="$name" /> <xsl:text>(</xsl:text> <xsl:apply-templates /> <xsl:text>)</xsl:text>
      </xsl:when>
      
      <!-- OUTPUT -->
      <xsl:when test="$name = 'out'">
        <xsl:value-of select="$name" /> <xsl:text>(</xsl:text>
        <xsl:apply-templates />
        <xsl:text>)</xsl:text>
      </xsl:when>
      
      <!-- NEW NAME -->
      <xsl:when test="$name = 'new'">
        <xsl:value-of select="$name" /> <xsl:text>(</xsl:text>
        <xsl:variable name="new-name" select="xpe:identifier" />
        <xsl:value-of select="$new-name/@name" />
        <xsl:text> : </xsl:text>
        <xsl:call-template name="F_types"> <xsl:with-param name="name" select="$new-name" /> </xsl:call-template>
        <xsl:text>)</xsl:text>
      </xsl:when>
      
      <!-- ENCRYPT: POSH RESPONSE - SOPH (SOSH) CHALLENGE - SOSH RESPONSE - (A)SYMMETRIC ENCRYPTION -->
      <xsl:when test="$name = 'encrypt'">
        <xsl:variable name="msg" select="xpe:message" />
        <xsl:variable name="nonce" select="$msg/xpe:identifier[@tag != 'id' and not(contains(@tag, 'auth'))]" />
        <xsl:if test="count($nonce) > 0 and (count($nonce) = 2 or not(contains($nonce, '?')))">
          <xsl:text>cast </xsl:text>
          <xsl:variable name="nonce-id">
            <xsl:choose>
              <xsl:when test="count($nonce) = 1"> <xsl:value-of select="$nonce/@name" /> </xsl:when>
              <xsl:otherwise> <xsl:value-of select="$nonce[not(contains(@tag, '?'))]/@name" /> </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>
          <xsl:value-of select="$nonce-id" />
          <xsl:text> is n</xsl:text> <xsl:value-of select="../@label" /> <xsl:text>_</xsl:text>
          <xsl:value-of select="$nonce-id" />
          <xsl:text> : </xsl:text>
          <xsl:call-template name="nonce-type">
            <xsl:with-param name="msg" select="$msg" />
            <xsl:with-param name="prefix" select="false()" />
          </xsl:call-template>
        </xsl:if>
      </xsl:when>
      
      <!-- DECRYPT: POSH RESPONSE - SOPH (SOSH) CHALLENGE - SOSH RESPONSE - (A)SYMMETRIC DECRYPTION -->
      <xsl:when test="$name = 'decrypt'">
        <xsl:variable name="msg" select="xpe:message" />
        <xsl:variable name="nonce" select="$msg/xpe:identifier[@tag != 'id' and not(contains(@tag, 'auth'))]" />
        <xsl:variable name="check" select="count($nonce) = 2 or (count($nonce) = 1 and not(contains($nonce/@tag, '?')))" />
        <xsl:variable name="nonce-id">
          <xsl:choose>
            <xsl:when test="count($nonce) = 1"> <xsl:value-of select="$nonce/@name" /> </xsl:when>
            <xsl:otherwise> <xsl:value-of select="$nonce[not(contains(@tag, '?'))]/@name" /> </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:value-of select="$name" /> <xsl:text> </xsl:text> <xsl:value-of select="$msg/@name" /> <xsl:text> as {</xsl:text>
        <xsl:call-template name="vbar" /> <xsl:text>msg_</xsl:text>
        <xsl:value-of select="../@label" /> <xsl:text>(</xsl:text>
        <xsl:for-each select="$msg//xpe:identifier">
          <xsl:if test="$check and @name = $nonce-id"> <xsl:text>x_</xsl:text> </xsl:if>
          <xsl:value-of select="@name" />
          <xsl:call-template name="comma" />
        </xsl:for-each>
        <xsl:text>)</xsl:text> <xsl:call-template name="vbar" /> <xsl:text>}_</xsl:text>
        <xsl:call-template name="key"> <xsl:with-param name="msg" select="$msg" /> </xsl:call-template>
        <xsl:if test="$check">
<xsl:text>.
  check x_</xsl:text> <xsl:value-of select="$nonce-id" /> <xsl:text> is </xsl:text> <xsl:value-of select="$nonce-id" />
        </xsl:if>
      </xsl:when>
      
      <!-- RUN - SOPH RESPONSE -->
      <xsl:when test="$name = 'run'">
        <xsl:text>begin(</xsl:text> <xsl:apply-templates /> <xsl:text>)</xsl:text>
        <xsl:variable name="rule" select="../xpe:rule" />
        <xsl:if test="$rule/@name != 'RUN'">
          <xsl:variable name="nonce" select="$rule//xpe:instruction[@name = 'nonce']/preceding-sibling::xpe:identifier/@name" />
<xsl:text>.
  cast </xsl:text> <xsl:value-of select="$nonce" /> <xsl:text> is (x_</xsl:text>
          <xsl:value-of select="$nonce" /> <xsl:text>: Public Response [])</xsl:text>
        </xsl:if>
      </xsl:when>
      
      <!-- COMMIT -->
      <xsl:when test="$name = 'commit'">
        <xsl:text>end(</xsl:text> <xsl:apply-templates /> <xsl:text>)</xsl:text>
      </xsl:when>
      
    </xsl:choose>
  </xsl:template>

  <!-- NIL -->
  <xsl:template match="xpe:nil" />

<!-- data -->
  
  <xsl:template match="xpe:valued-identifier" />
  
  <xsl:template match="xpe:tagged-identifiers">
    <xsl:apply-templates />
    <xsl:call-template name="comma" />
  </xsl:template>
  
  <xsl:template match="xpe:tag-parameters">
    <xsl:text>(</xsl:text> <xsl:apply-templates /> <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="xpe:identifier">
    <xsl:param name="show-tag" select="false()" />
    <xsl:variable name="subst">
      <xsl:call-template name="subst"> <xsl:with-param name="data" select="@name" /> </xsl:call-template>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="$subst != ''">
        <xsl:value-of select="$subst" />
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="@name" />
        <xsl:if test="$show-tag and @tag"> <xsl:text> : </xsl:text> <xsl:value-of select="@tag" /> </xsl:if>
        <xsl:apply-templates />
      </xsl:otherwise>
    </xsl:choose>
    <xsl:call-template name="comma" />
  </xsl:template>
  
  <xsl:template name="subst">
    <xsl:param name="data" />
    <!-- preceding-sibling ... -->
    <xsl:for-each select="ancestor::xpe:instruction-node/preceding-sibling::xpe:instruction-node/xpe:instruction[@name = 'encrypt' or @name = 'run']">
      <xsl:call-template name="subst-ids"> <xsl:with-param name="data" select="$data" /> </xsl:call-template>
    </xsl:for-each>
    <!-- ... or-self -->
    <xsl:for-each select="ancestor::xpe:instruction-node/xpe:instruction[@name = 'encrypt' or @name = 'run']">
      <xsl:call-template name="subst-ids"> <xsl:with-param name="data" select="$data" /> </xsl:call-template> 
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template name="subst-ids">
    <xsl:param name="data" />
      <xsl:variable name="msg" select="xpe:message" />
      <xsl:variable name="msg-nonce" select="$msg/xpe:identifier[@tag != 'id' and not(contains(@tag, 'auth'))]" />
      <xsl:choose>
        <!-- encrypt -->
        <xsl:when test="$msg/@name = $data">
          <xsl:for-each select="$msg"> <xsl:call-template name="message" /> </xsl:for-each>
        </xsl:when>
        <xsl:when test="count($msg-nonce) > 0 and (count($msg-nonce) = 2 or not(contains($msg-nonce, '?'))) and $msg-nonce/@name = $data">
          <xsl:text>n</xsl:text> <xsl:value-of select="ancestor::xpe:instruction-node/@label" /> 
          <xsl:text>_</xsl:text> <xsl:value-of select="$data" /> 
        </xsl:when>
        <!-- run -->
        <xsl:when test="@name = 'run'">
          <xsl:variable name="rule" select="../xpe:rule" />
          <xsl:if test="$rule/@name != 'RUN'">
            <xsl:variable name="nonce" select="$rule//xpe:instruction[@name = 'nonce']/preceding-sibling::xpe:identifier/@name" />
            <xsl:if test="$nonce = $data"> <xsl:text>x_</xsl:text> <xsl:value-of select="$nonce" /> </xsl:if>
          </xsl:if>
        </xsl:when>
      </xsl:choose>
  </xsl:template>
  
  <xsl:template name="message" match="xpe:message">
    <xsl:text>{</xsl:text> <xsl:call-template name="vbar" /> <xsl:text>msg_</xsl:text> <xsl:value-of select="ancestor::xpe:instruction-node/@label" />
    <xsl:text>(</xsl:text> <xsl:apply-templates /> <xsl:text>)</xsl:text> <xsl:call-template name="vbar" /> <xsl:text>}</xsl:text>
    <xsl:if test="@encryption != 'none' and @key != ''"> <xsl:text>_</xsl:text> <xsl:call-template name="key" /> </xsl:if>
  </xsl:template>
  
  <xsl:template name="key">
    <xsl:param name="msg" select="." />
    <xsl:choose>
      <xsl:when test="$msg/@encryption = 'public'"> <xsl:text>Pub(</xsl:text> <xsl:value-of select="$msg/@key" /> <xsl:text>)</xsl:text> </xsl:when>
      <xsl:when test="$msg/@encryption = 'private'"> <xsl:text>Priv(</xsl:text> <xsl:value-of select="$msg/@key" /> <xsl:text>)</xsl:text> </xsl:when>
      <xsl:when test="$msg/@encryption = 'symmetric'"> <xsl:value-of select="$msg/@key" /> </xsl:when>
    </xsl:choose>
  </xsl:template>

<!-- tags2type -->

  <xsl:template name="tags2type">
    <xsl:param name="key" />
    <xsl:param name="key-type" />
    <xsl:param name="label" />
    <xsl:variable name="msg" select="/.//xpe:instruction-node[@label = $label]/xpe:instruction[@name = 'encrypt']/xpe:message[@key = $key]" />
    <xsl:variable  name="ex-type" select="$msg/xpe:identifier[@tag != 'id' and not(contains(@tag, 'auth'))]/@tag" />
    <xsl:text>msg_</xsl:text> <xsl:value-of select="$label" /> <xsl:text>(</xsl:text>    
    <xsl:for-each select="$msg/xpe:identifier">
      <xsl:call-template name="fresh-id"> <xsl:with-param name="id" select="@name" /> <xsl:with-param name="label" select="$label" /> </xsl:call-template>
      <xsl:text> : </xsl:text>
      <xsl:choose>
        <xsl:when test="@tag = $ex-type">
          <xsl:call-template name="nonce-type">
            <xsl:with-param name="msg" select="$msg" />
            <xsl:with-param name="key-type" select="$key-type" />
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise> <xsl:text>Un</xsl:text> </xsl:otherwise>
      </xsl:choose>
      <xsl:call-template name="comma" />
    </xsl:for-each>
    <xsl:text>)</xsl:text>
  </xsl:template>
  
  <xsl:template name="fresh-id">
    <xsl:param name="id" />
    <xsl:param name="label" />
    <xsl:text>x</xsl:text> <xsl:value-of select="$label" /> <xsl:text>_</xsl:text> <xsl:value-of select="$id" />
  </xsl:template>
  
  <xsl:template name="nonce-type">
    <xsl:param name="msg" />
    <xsl:param name="key-type" select="ancestor::xpe:sequential-process//xpe:declaration[@identifier = $msg/@key]/xpe:key-type" />
    <xsl:param name="prefix" select="true()" />
    
    <xsl:variable  name="ex-type" select="$msg/xpe:identifier[@tag != 'id' and not(contains(@tag, 'auth'))]/@tag" />
    <xsl:variable  name="auth-set" select="$msg/*[@tag = 'auth']" />
    <xsl:variable  name="conf-set" select="$msg/*[@tag = 'auth_conf']" />
    <xsl:variable name="sosh-res" select="count($conf-set)  > 0" />
    <xsl:variable name="id" select="$msg/xpe:identifier[@tag = 'id']/@name" />
    <xsl:variable name="msg-type">
      <xsl:choose>
        <xsl:when test="$sosh-res or not(contains($ex-type, '?'))">Response</xsl:when>
        <xsl:otherwise>Challenge</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
      <xsl:variable name="msg-privacy">
        <xsl:choose>
          <xsl:when test="contains(@tag , 'SOSH')">Private</xsl:when>
          <xsl:otherwise>Public</xsl:otherwise>
        </xsl:choose>
      </xsl:variable>

    <xsl:choose>
      <xsl:when test="$sosh-res and contains(@tag, '?')">
        <xsl:value-of select="$msg-privacy" /> <xsl:text> Challenge [end(</xsl:text>
        <xsl:call-template name="effect-ids"> <xsl:with-param name="id" select="$id" /> <xsl:with-param name="key-type" select="$key-type" /> </xsl:call-template>
        <xsl:value-of select="$id" /> <xsl:text>, </xsl:text>
        <xsl:for-each select="$conf-set"> <xsl:call-template name="auth-id"> <xsl:with-param name="prefix" select="$prefix" /> </xsl:call-template> </xsl:for-each>
        <xsl:text>) + end (</xsl:text>
        <xsl:call-template name="effect-ids"> <xsl:with-param name="id" select="$id" /> <xsl:with-param name="key-type" select="$key-type" /> </xsl:call-template>
        <xsl:value-of select="$id" /> <xsl:text>, </xsl:text>
        <xsl:for-each select="$auth-set"> <xsl:call-template name="auth-id"> <xsl:with-param name="prefix" select="$prefix" /> </xsl:call-template> </xsl:for-each>
        <xsl:text>)]</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$msg-privacy" /> <xsl:text> </xsl:text> <xsl:value-of select="$msg-type" /> <xsl:text> [end(</xsl:text>
        <xsl:choose>
          <xsl:when test="$sosh-res">
            <xsl:value-of select="$id" /> <xsl:text>, </xsl:text>
            <xsl:call-template name="effect-ids"> <xsl:with-param name="id" select="$id" /> <xsl:with-param name="key-type" select="$key-type" /> </xsl:call-template>
            <xsl:for-each select="$conf-set"> <xsl:call-template name="auth-id"> <xsl:with-param name="prefix" select="$prefix" /> </xsl:call-template> </xsl:for-each>
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="effect-ids"> <xsl:with-param name="id" select="$id" /> <xsl:with-param name="key-type" select="$key-type" /> </xsl:call-template>
            <xsl:value-of select="$id" /> <xsl:text>, </xsl:text>
            <xsl:for-each select="$auth-set"> <xsl:call-template name="auth-id"> <xsl:with-param name="prefix" select="$prefix" /> </xsl:call-template> </xsl:for-each>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:text>)]</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="auth-id">
    <xsl:param name="prefix" />
    <xsl:choose>
      <xsl:when test="@name"> <xsl:call-template name="id"> <xsl:with-param name="prefix"  select="$prefix" /> </xsl:call-template> </xsl:when>
      <xsl:otherwise>
        <xsl:for-each select="xpe:identifier"> <xsl:call-template name="id"> <xsl:with-param name="prefix"  select="$prefix" /> </xsl:call-template> </xsl:for-each>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="id">
    <xsl:param name="prefix" />
    <xsl:if test="$prefix">
      <xsl:text>x</xsl:text>
      <xsl:value-of select="ancestor::xpe:instruction-node/@label" />
      <xsl:text>_</xsl:text>
    </xsl:if>
    <xsl:value-of select="@name" />
    <xsl:call-template name="comma" /> 
  </xsl:template>

  <xsl:template name="effect-ids">
    <xsl:param name="id" />
    <xsl:param name="key-type" />
    <xsl:for-each select="$key-type/xpe:identifier">
      <xsl:if test="@name != $id"> <xsl:value-of select="@name" /> <xsl:text>, </xsl:text> </xsl:if>
    </xsl:for-each>
  </xsl:template>

<!-- F_key -->

  <xsl:template name="F_key">
    <xsl:param name="key" />
    <xsl:param name="key-type" />
    <xsl:variable name="ciph" select="/.//xpe:instruction-node[@label]/xpe:instruction[@name = 'encrypt']/xpe:message[@key = $key]/../.." /> <!-- ciph(k,P) -->
    <xsl:if test="$key-type/@type = 'symmetric'"> <xsl:text>Shared</xsl:text> </xsl:if> <xsl:text>Key(</xsl:text>
    <xsl:choose>
      <xsl:when test="count($ciph) > 0">
        <xsl:for-each select="$ciph">
          <xsl:call-template name="tags2type">
            <xsl:with-param name="key" select="$key" />
            <xsl:with-param name="key-type" select="$key-type" />
            <xsl:with-param name="label" select="@label" />
          </xsl:call-template>
          <xsl:call-template name="vert" />
        </xsl:for-each>
      </xsl:when>
      <xsl:otherwise> <xsl:text>Top</xsl:text> </xsl:otherwise>
    </xsl:choose>
    <xsl:text>)</xsl:text>
  </xsl:template>
  
<!-- F_types -->

  <xsl:template name="F_types">
    <xsl:param name="name" />
    <xsl:variable name="gamma" select="..//xpe:container[@name = 'gamma']" />
    <xsl:choose>
      <xsl:when test="count($gamma/*) = 0"> <xsl:text>fi(</xsl:text> <xsl:value-of select="$P" /> <xsl:text></xsl:text>) : Un</xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="F_msg">
          <xsl:with-param name="type-id" select="$name" />
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
<!-- F_msg -->

    <xsl:template name="F_msg">
      <xsl:param name="type-id" />
      <xsl:variable name="type-node" select="$type-id/ancestor::xpe:instruction-node//xpe:actions//xpe:identifier[@name = $type-id/@name]/following-sibling::*" />
      <xsl:variable name="type" select="$type-node/@name" />
      <xsl:choose>
        <xsl:when test="$type = 'Un'"> <xsl:text>Un</xsl:text> </xsl:when>
        <xsl:otherwise>
          <xsl:choose>
            <xsl:when test="contains($type, 'nonce_public') or contains($type, 'nonce_secpub')"> <xsl:text>Public</xsl:text> </xsl:when>
            <xsl:when test="contains($type, 'nonce_secret')"> <xsl:text>Private</xsl:text> </xsl:when>
            <xsl:otherwise> <xsl:text>Private</xsl:text> </xsl:otherwise>
          </xsl:choose>
          <xsl:text> Challenge [end(</xsl:text>
          <xsl:for-each select="$type-node"> <xsl:apply-templates /> </xsl:for-each>
          <xsl:text>)]</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:template>

<!-- utility templates -->

  <xsl:template name="comma"> <xsl:if test="position() != last()"> <xsl:text>, </xsl:text> </xsl:if> </xsl:template>
  
  <xsl:template name="vert"> <xsl:if test="position() != last()"> <xsl:text> | </xsl:text> </xsl:if> </xsl:template>

  <xsl:template name="vbar">
    <xsl:if test="../descendant-or-self::xpe:message/@encryption = 'public' or @encryption = 'private'"> <xsl:text>|</xsl:text> </xsl:if>
  </xsl:template>

</xsl:stylesheet>
