/*
 * ETOperation.java
 *
 * Created on 30 settembre 2004, 18.38
 */

package it.unive.dsi.pear.core.evaluationtree;

import it.unive.dsi.pear.core.parser.Container;
import it.unive.dsi.pear.core.parser.Data;
import it.unive.dsi.pear.core.parser.Operation;
import it.unive.dsi.pear.core.parser.OperationSourceType;
import it.unive.dsi.pear.core.parser.ValidationRulesNode;

/**
 * This class implements the evaluation tree operation node.
 * @author Francesco Placella
 */
public class ETOperation extends ETOpNode implements Operation
{
    private static final int DEFAULT_BUF_SIZE = 50;
    
    private Data[] substArgs;
    private boolean instance = false;

    public ETOperation(Operation data) { super(data); }
    
    protected final Operation getData()
    {
        return (Operation)this.getDataNode();
    }
    
    protected ETNode getChildInstance(ValidationRulesNode node) { return null; }
    protected void buildEvaluationTree(ValidationRulesNode node)
    {
        this.setDataNode(node);
    }
    
    /** Obtains the last instantiation recorded. */
    public ETNode getLastInstantiation()
    {
        ETOperation cloned = (ETOperation)this.cloneNode();
        cloned.instance = true;
        instance = false;
        substArgs = null;
        return cloned;
    }

// ----------------------------------------------------------------------------
    
    /* the methods below get their data from the related data tree. */
    
    public final String toString()
    {
        return this.toString(this.getContainerName(), this.getArguments());
    }
    
    public String toString(String containerName, Object[] arguments)
    {
        StringBuffer res = new StringBuffer(DEFAULT_BUF_SIZE);
        res.
            append("(").append(this.getDefSrcType()).append(") ").
            append(containerName).append(".").append(this.getOperationName()).
            append("(");
        
        if (arguments != null)
        {
            int size = arguments.length;
            if (size > 0)
            {
                res.append(arguments[0]);
                for (int i = 1; i < size; ++i)
                    res.append(", " + arguments[i]);
            }
        }

        res.append(")");
        return res.toString();
    }
    
    /** Exploits the given <CODE>Evaluator</CODE> to get the operation result.
     */
    public boolean evaluate(EvalInfo evalInfo)
        throws Container.OperationException
    {
        this.buildPath(evalInfo);
        
        boolean res;
        try
        {
            evalInfo.setOperation(this);
            res = evalInfo.getEvaluator().evaluate(evalInfo);
            substArgs = evalInfo.getInstantiatedArguments();
        }
        finally { this.destroyPath(evalInfo); }
        
        return res;
    }
    
    public final String getContainerName() 
    {
        return this.getData().getContainerName();
    }

    public final String getOperationName()
    {
        return this.getData().getOperationName();
    }

    public final Data[] getArguments()
    {
        return instance && substArgs != null ? substArgs : this.getData().getArguments();
    }

    public final boolean isNative()
    {
        return this.getData().isNative();
    }
}
